import copy
from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.Newswire import Newswire
from src.cs_models.resources.Subsidiary import Subsidiary
from src.cs_models.resources.CompanySEC import CompanySEC
from src.cs_models.resources.CompanyOUS import CompanyOUS
from src.cs_models.resources.File import File


class NewswireResourceTestCase(TestCase):
    def setUp(self):
        super(NewswireResourceTestCase, self).setUp()
        self.inst = Newswire()
        self.inst_file = File()
        self.inst_subsidiary = Subsidiary()
        self.inst_sec_company = CompanySEC()
        self.inst_ous_company = CompanyOUS()

        self.company_sec1 = self.inst_sec_company.create({
            'cik_str': '1382101',
            'ticker': 'STRO',
            "title": "SUTRO BIOPHARMA, INC.",
        })

        self.company_sec2 = self.inst_sec_company.create({
            "cik_str": "885590",
            "ticker": "BHC",
            "title": "Bausch Health Companies Inc."
        })

        self.company_ous1 = self.inst_ous_company.create({
            'name': 'Kolon Tissuegene',
            'ticker': None,
            'exchange': 'NSE',
        })

        self.subsidiary1 = self.inst_subsidiary.create({
            'name': 'Sutro Biopharma Inc.',
            'company_sec_id': self.company_sec1["id"],
            'company_ous_id': None,
        })

        self.subsidiary2 = self.inst_subsidiary.create({
            'name': 'Bausch Health Inc.',
            'company_sec_id': self.company_sec2["id"],
            'company_ous_id': None,
        })

        self.subsidiary3 = self.inst_subsidiary.create({
            'name': 'Kolon Tissuegene',
            'company_sec_id': None,
            'company_ous_id': self.company_ous1["id"],
        })

        self.file1 = self.inst_file.create(
            {
                "file_format": "xml",
                "s3_bucket_name": "newswires",
                "s3_key_name": "PRN/202004131700PR_NEWS_USPR_____NY76479.xml",
            }
        )

        self.file2 = self.inst_file.create(
            {
                "file_format": "xml",
                "s3_bucket_name": "newswires",
                "s3_key_name": "PRN/20200413170343R_NEWS_USPR_____NY76349.xml",
            }
        )

        self.newswire1 = self.inst.create({
            'date': '20200413T152300',
            'source': 'PR Newswire',
            'headline': 'Sutro Biopharma to Present Updated Clinical Data '
                        'for its STRO-002 Antibody-Drug Conjugate at the '
                        'AACR Virtual Annual Meeting on April 27, 2020',
            'drugs': '[STRO-002]',
            'conditions': "['Levine Cancer', 'ovarian and endometrial "
                          "cancer', 'ovarian cancer', "
                          "'Platinum-Resistant/Refractory Epithelial Ovarian Cancer', "
                          "'Cancers']",
            "subsidiary_id": self.subsidiary1["id"],
            "news_file_id": self.file1["id"]
        })

        self.newswire2 = self.inst.create({
            'date': '20200409T120300',
            'source': 'PR Newswire',
            'headline': 'Bausch Health Initiates VIRAZOLE® (Ribavirin for '
                        'Inhalation Solution, USP) Clinical Study in '
                        'Patients with COVID-19',
            'drugs': "['VIRAZOLE', 'Ribavirin']",
            'conditions': "['respiratory distress', 'COVID-19']",
            "subsidiary_id": self.subsidiary2["id"],
            "news_file_id": self.file2["id"]
        })

        self.newswire3 = self.inst.create({
            'date': '20200413T063000',
            'source': 'PR Newswire',
            'headline': 'Kolon Tissuegene Announces Plans To Resume US Phase '
                        'III Clinical Trial For TG-C',
            'drugs': "['TG-C']",
            'conditions': "['osteoarthritis', 'COVID-19']",
            "subsidiary_id": self.subsidiary3["id"],
            "news_file_id": self.file2["id"]
        })

        self.valid_data = {
            'date': '20200410T080000',
            'source': 'PR Newswire',
            'headline': 'FDA Approves First Ever Treatment for '
                        'Neurofibromatosis',
            'drugs': "['Koselugo', 'selumetinib']",
            'conditions': "['neurofibromatosis']",
            "subsidiary_id": self.subsidiary1["id"],
            "news_file_id": self.file1["id"]
        }

    @parameterized.expand([
        ('source',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        expected_data = {
            **self.valid_data,
            **{
                'date': '2020-04-10T08:00:00+00:00',
            }
        }
        second_equals_first(expected_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

    @parameterized.expand([
        ('id', 'newswire1', 'id', 1),
        ('headline', 'newswire1', 'headline', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('headline',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.newswire1[field_name],
        })
        second_equals_first(
            self.newswire1,
            resp,
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'subsidiary_id': self.valid_data['subsidiary_id'],
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(3, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(4, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            **self.valid_data,
            **{'headline': self.newswire1['headline'],
               'subsidiary_id': self.subsidiary1['id'],
               },
        }
        resp = self.inst.upsert(data)
        expected_data = {
            **data,
            **{
                'date': '2020-04-10T08:00:00+00:00',
            }
        }
        second_equals_first(
            expected_data,
            resp,
        )
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_with_invalid_subsidiary_id(self):
        invalid_id = 9999999
        data = {
            **self.valid_data,
            **{'headline': self.newswire1['headline'],
               'subsidiary_id': invalid_id,
               },
        }
        self.assertRaises(
            IntegrityError,
            self.inst.upsert,
            data,
        )



from datetime import datetime
from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
)

from ...database import Base


class FederalCaseRefModel(Base):
    __tablename__ = "federal_case_refs"

    id = Column(Integer, primary_key=True)
    case_name = Column(String(128), nullable=False)
    volume = Column(Integer, nullable=False)
    reporter_type = Column(String(50), nullable=False)
    page = Column(Integer, nullable=False)
    title = Column(String(128), nullable=False)
    link = Column(String(128), nullable=False)
    federal_case_file_id = Column(Integer, ForeignKey("files.id"))

    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from marshmallow import ValidationError
from parameterized import parameterized

from test.mixins.BasicDataMixin import BasicDataMixin
from src.cs_models.resources.PriorArt import PriorArt
from test.backendtestcase import TestCase
from test.utils import second_equals_first


class PriorArtResourceTestCase(BasicDataMixin, TestCase):
    def setUp(self):
        super(PriorArtResourceTestCase, self).setUp()
        self.inst = PriorArt()

    def test_create(self):
        data = {
            'ptab2_document_id': self.ptab2_document1['id'],
            'tag': 'Jackson',
            'title': 'U.S 1234',
            'exhibit': 'EX 1010',
        }
        resp = self.inst.create(data)
        second_equals_first(
            data,
            resp,
        )

    def test_create_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {},
        )

    def test_create_validation_error_blank_tag(self):
        data = {
            'ptab2_document_id': self.ptab2_document1['id'],
            'tag': '',
            'title': 'U.S 1234',
            'exhibit': 'EX 1010',
        }
        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

    @parameterized.expand([
        ('ptab2_document_id', 'prior_art1', 'ptab2_document_id', 'prior_art1'),
        ('tag', 'prior_art2', 'tag', 'prior_art2'),
    ])
    def test_read_w_params(
        self,
        query_field,
        attr1,
        attr2,
        expected_prior_art_attr,
    ):
        resp = self.inst.read({
            query_field: getattr(self, attr1)[attr2],
        })
        self.assertEqual(1, len(resp))
        expected_prior_art = getattr(self, expected_prior_art_attr)
        self.assertEqual(
            expected_prior_art['id'],
            resp[0]['id'],
        )

    def test_update(self):
        new_data = {
            'ptab2_document_id': 9999,
            'tag': 'what what',
            'title': 'some some',
        }
        resp = self.inst.update(
            id=self.prior_art1['id'],
            params=new_data,
        )
        second_equals_first(
            {
                'id': self.prior_art1['id'],
                'ptab2_document_id': self.prior_art1['ptab2_document_id'],
                'tag': self.prior_art1['tag'],
                'title': new_data['title'],
            },
            resp,
        )

    def test_delete(self):
        response = self.inst.read({'id': self.prior_art1['id']})
        self.assertEqual(len(response), 1)
        response = self.inst.delete(id=self.prior_art1['id'])
        self.assertEqual(response, 'Successfully deleted')
        response = self.inst.read({'id': self.prior_art1['id']})
        self.assertEqual(len(response), 0)

    def test_upsert_raises_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {},
        )

    def test_upsert_creates_a_new_prior_art(self):
        self.assertEqual(2, len(self.inst.read({})))
        data = {
            'ptab2_document_id': self.ptab2_document1['id'],
            'tag': 'Jackson',
            'title': 'U.S 1234',
            'exhibit': '',
        }
        resp = self.inst.upsert(data)
        second_equals_first(
            data,
            resp,
        )
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_existing(self):
        self.assertEqual(2, len(self.inst.read({})))
        data = {
            'ptab2_document_id': self.prior_art1['ptab2_document_id'],
            'tag': self.prior_art1['tag'],
            'title': 'Something new',
            'exhibit': 'EX 2123',
        }
        resp = self.inst.upsert(data)
        second_equals_first(
            data,
            resp,
        )
        self.assertEqual(2, len(self.inst.read({})))



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
    ValidationError,
)

from ...utils.utils import convert_string_to_datetime


def pre_load_helper(in_data):
    try:
        if in_data.get('patent_expire_date'):
            in_data['patent_expire_date'] = convert_string_to_datetime(
                date=in_data['patent_expire_date'],
                string_format='%b %d, %Y',
            )
        else:
            in_data['patent_expire_date'] = None
        if in_data.get('submission_date'):
            in_data['submission_date'] = convert_string_to_datetime(
                date=in_data['submission_date'],
                string_format='%b %d, %Y',
            )
        else:
            in_data['submission_date'] = None
    except (TypeError, ValueError):
        raise ValidationError('Invalid date format')
    return in_data


class OrangeBookPatentResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    appl_no = fields.String(required=True)
    appl_type = fields.String(required=True, validate=not_blank)
    delist_flag = fields.String(required=True)
    drug_product_flag = fields.String(required=True)
    drug_substance_flag = fields.String(required=True)
    patent_no = fields.String(required=True, validate=not_blank)
    patent_use_code = fields.String(required=True, validate=not_blank)
    product_no = fields.String(required=True, validate=not_blank)
    patent_expire_date = fields.DateTime(required=True, allow_none=True)
    submission_date = fields.DateTime(required=True, allow_none=True)

    @pre_load
    def pre_load_date_fields(self, in_data):
        return pre_load_helper(in_data)


class OrangeBookPatentQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    appl_type = fields.String(validate=not_blank)
    appl_no = fields.String(validate=not_blank)
    appl_nos = fields.List(fields.Integer())
    patent_no = fields.String(validate=not_blank)
    product_no = fields.String(validate=not_blank)
    patent_use_code = fields.String(validate=not_blank)


class OrangeBookPatentPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    appl_no = fields.String(validate=not_blank)
    appl_type = fields.String(validate=not_blank)
    delist_flag = fields.String()
    drug_product_flag = fields.String()
    drug_substance_flag = fields.String()
    patent_no = fields.String(validate=not_blank)
    patent_use_code = fields.String(validate=not_blank)
    product_no = fields.String(validate=not_blank)
    patent_expire_date = fields.DateTime(allow_none=True)
    submission_date = fields.DateTime(allow_none=True)

    @pre_load
    def pre_load_date_fields(self, in_data):
        return pre_load_helper(in_data)



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
    ForeignKey,
)

from ...database import Base


class SetIDPharmacologicCrossModel(Base):
    __tablename__ = "set_id_pharmacologic"

    spl_set_id = Column(String(128), nullable=False, primary_key=True)
    pharma_set_id = Column(String(128), nullable=False, index=True)



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
    ForeignKey,
    UniqueConstraint,
)
from datetime import datetime
from ...database import Base


class ClaimChallengedModel(Base):
    __tablename__ = 'claims_challenged'

    id = Column(Integer, primary_key=True)
    ptab2_proceeding_id = Column(
        Integer,
        ForeignKey('ptab2_proceedings.id'),
        nullable=False,
    )
    claim_id = Column(
        Integer,
        ForeignKey('claims.id'),
        nullable=False,
    )
    prior_art_combination = Column(
        Integer,
        nullable=False,
    )
    prior_art_id = Column(
        Integer,
        ForeignKey('prior_arts.id'),
        nullable=False,
    )
    prior_art_nature = Column(String(128))
    nature_of_challenge = Column(Text, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )
    __table_args__ = (
        UniqueConstraint(
            'ptab2_proceeding_id',
            'claim_id',
            'prior_art_combination',
            'prior_art_id',
        ),
    )



from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime

from ...database import db_session
from ...utils.utils import update_model
from ...resources.Patent.models import PatentModel
from .schemas import (
    PatentResourceSchema,
    PatentQueryParamsSchema,
    PatentPatchSchema,
)
from marshmallow import ValidationError


schema_resource = PatentResourceSchema()
schema_params = PatentQueryParamsSchema()
schema_patch = PatentPatchSchema()


class DBException(SQLAlchemyError):
    pass


class PatentNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class Patent:
    @staticmethod
    def create(params):
        """
        :param
            Dict: PatentResourceSchema
        :return:
            PatentResourceSchema
        :exception:
            ValidationError
            DBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            params: PatentQueryParamsSchema
        :return:
            List<PatentResourceSchema>
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        patent_query = _build_query(params=data)
        response = schema_resource.dump(patent_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        :param
            params: PatentQueryParamsSchema
        :return:
            PatentResourceSchema
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        patent_query = _build_query(params=data).one()
        response = schema_resource.dump(patent_query).data
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            PatentNotFoundException
            DBException
        """

        patent_query = db_session.query(PatentModel).filter_by(id=id).first()
        if not patent_query:
            raise PatentNotFoundException('Patent does not exist!')
        try:
            db_session.delete(patent_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise DBException('DB error')

    @staticmethod
    def upsert(params):
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        patent_query = db_session.query(PatentModel).filter_by(
            patent_number=params.get('patent_number'),
            jurisdiction=params.get('jurisdiction'),
        ).first()

        if not patent_query:
            response = _helper_create(data)
        else:
            response = _helper_update(data, patent_query)
        return response

    @staticmethod
    def update(id, params):
        """

        Args:
            id: int
            params: PatentPatchSchema

        Returns:
            PatentResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        pacer_case_query = db_session.query(
            PatentModel).filter_by(id=id).one()
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, pacer_case_query)
        return response

    @staticmethod
    def pharma_patents():
        # pharma patents have app_grp_art_number in the 1600s
        patent_query = db_session.query(PatentModel).filter(
            PatentModel.app_grp_art_number.op('regexp')('^16\d\d$'))
        response = schema_resource.dump(patent_query, many=True).data
        return response


def _helper_create(data):
    new_patent = PatentModel(
        updated_at=datetime.utcnow(),
        **data,
    )
    try:
        db_session.add(new_patent)
        db_session.commit()
        patent_query = db_session.query(PatentModel).get(new_patent.id)
        response = schema_resource.dump(patent_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, patent_query):
    data['id'] = patent_query.id
    data['patent_number'] = patent_query.patent_number
    data['jurisdiction'] = patent_query.jurisdiction
    try:
        update_model(data, patent_query)
        db_session.commit()
        response = schema_resource.dump(patent_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(PatentModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('patent_number'):
        q = q.filter_by(patent_number=params.get('patent_number'))
    if params.get('jurisdiction'):
        q = q.filter_by(jurisdiction=params.get('jurisdiction'))
    if params.get('app_grp_art_number'):
        q = q.filter_by(app_grp_art_number=params.get('app_grp_art_number'))
    if params.get('primary_identifier'):
        q = q.filter_by(primary_identifier=params.get('primary_identifier'))
    return q



import copy

from test.backendtestcase import TestCase
from test.utils import second_equals_first

from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import NoResultFound
from parameterized import parameterized

from src.cs_models.resources.PTAB2Document import PTAB2Document
from src.cs_models.resources.DocumentReference import DocumentReference
from src.cs_models.resources.File import File


class DocumentReferenceResourceTestCase(TestCase):
    def setUp(self):
        super(DocumentReferenceResourceTestCase, self).setUp()

        self.inst_file = File()
        self.inst_ptab2_document = PTAB2Document()
        self.inst = DocumentReference()

        self.ptab2_document1 = self.inst_ptab2_document.create(
            {
                "document_category": "Exhibits",
                "document_filing_date": "05-08-2019",
                "document_identifier": "1000",
                "document_name": "Exhibit 1066 - EC-Naprosyn 2004 label.pdf",
                "document_number": "1066",
                "document_size": "-",
                "document_title_text": "EC Naprosyn 2004 label",
                "document_type_name": "EXHIBIT",
                "filing_party_category": "-",
                "media_type_category": "-",
                "petitioner_application_number_text": "-",
                "petitioner_counsel_name": "Alan H. Pollack",
                "petitioner_grant_date": "-",
                "petitioner_group_art_unit_number": "-",
                "petitioner_inventor_name": "-",
                "petitioner_party_name": "Mylan Pharmaceuticals Inc.",
                "petitioner_patent_number": "-",
                "petitioner_patent_owner_name": "-",
                "petitioner_technology_center_number": "-",
                "proceeding_number": "IPR2018-00272",
                "proceeding_type_category": "AIA Trial",
                "respondent_application_number_text": "14980639",
                "respondent_counsel_name": "Thomas Blinka",
                "respondent_grant_date": "07-19-2016",
                "respondent_group_art_unit_number": "1617",
                "respondent_inventor_name": "-",
                "respondent_party_name": "Horizon Pharma USA, Inc.",
                "respondent_patent_number": "9393208",
                "respondent_patent_owner_name": "AULT et al",
                "respondent_technology_center_number": "1600",
                "subproceeding_type_category": "IPR",
            }
        )

        self.file1 = self.inst_file.create(
            {
                "file_format": "html",
                "s3_bucket_name": "federal-case-pages",
                "s3_key_name": "something.html",
            }
        )

        self.document_reference1 = self.inst.create(
            {
                "ptab2_document_id": self.ptab2_document1["id"],
                "type": "FEDERAL_CASE",
                "value": "1:16-cv-00469 (filed Feb. 26, 2016)",
                "federal_case_number": "123",
                "link": "www.abc.com",
                "pages": "[23, 78, 81]",
            }
        )

        self.document_reference2 = self.inst.create(
            {
                "ptab2_document_id": self.ptab2_document1["id"],
                "type": "CIVIL_CASE",
                "value": "Something else",
                "pages": "[12, 42, 55]",
            }
        )

        self.valid_data = {
            "ptab2_document_id": self.ptab2_document1["id"],
            "type": "FEDERAL_CASE",
            "value": "778 F.3d 1271, 1279",
            "pages": "[34, 35, 67, 73]",
        }

    @parameterized.expand(
        [("ptab2_document_id",), ("type",), ("value",),]
    )
    def test_create_validation_error_missing_field(self, field_to_pop):
        data = copy.copy(self.valid_data)
        data.pop(field_to_pop)
        self.assertRaises(
            ValidationError, self.inst.create, data,
        )

    def test_create_raises_exception_if_null_ptab2_document_id(self):
        invalid_ptab2_document_id = None
        data = {
            **self.valid_data,
            **{"ptab2_document_id": invalid_ptab2_document_id},
        }
        self.assertRaises(
            ValidationError, self.inst.create, data,
        )

    def test_create_violates_fk_constraint(self):
        invalid_ptab2_document_id = 12323423
        data = {
            **self.valid_data,
            **{"ptab2_document_id": invalid_ptab2_document_id},
        }
        self.assertRaises(
            IntegrityError, self.inst.create, data,
        )

    def test_create_violates_fk_constraint_file_id(self):
        invalid_federal_case_file_id = 12323423
        data = {
            **self.valid_data,
            **{"federal_case_file_id": invalid_federal_case_file_id},
        }
        self.assertRaises(
            IntegrityError, self.inst.create, data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        second_equals_first(self.valid_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

    @parameterized.expand(
        [
            ("ptab2_document_id", "ptab2_document1", "id", 2),
            ("type", "document_reference1", "type", 1),
            ("value", "document_reference1", "value", 1),
            (
                "federal_case_number",
                "document_reference1",
                "federal_case_number",
                1,
            ),
            (
                "link",
                "document_reference1",
                "link",
                1,
            ),
        ]
    )
    def test_read_w_params(
        self, field_name, attr, attr_field, expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

        resp = self.inst.read({field_name: getattr(self, attr)[attr_field],})
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand(
        [("id", 999, NoResultFound),]
    )
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception, self.inst.one, {field_name: field_value,},
        )

    @parameterized.expand(
        [("id",), ("type",), ("value",),]
    )
    def test_one(self, field_name):
        resp = self.inst.one(
            {field_name: self.document_reference1[field_name],}
        )
        second_equals_first(
            self.document_reference1, resp,
        )

    def test_update(self):
        new_data = {
            "value": "something new",
            "type": "ny",
            "federal_case_headline": "some headline",
            "federal_case_number": "case number 123",
            "link": "www.asd.com",
            "federal_case_file_id": self.file1["id"],
            "pages": "[23, 45, 46]",
        }
        resp = self.inst.update(
            id=self.document_reference1["id"], params=new_data,
        )
        expected_result = {
            **self.document_reference1,
            **new_data,
            # the following fields cannot be updated
            **{
                "value": self.document_reference1["value"],
                "type": self.document_reference1["type"],
            },
        }

        second_equals_first(
            expected_result, resp, fields_to_ignore=["updated_at"],
        )

    def test_create_without_pages_raises_exception(self):
        data = {
            **self.valid_data,
            **{"pages": None},
        }
        self.assertRaises(
            ValidationError, self.inst.create, data,
        )

    def test_update_without_pages_raises_exception(self):
        new_data = {
            "value": "something new",
            "type": "ny",
            "federal_case_headline": "some headline",
            "federal_case_number": "case number 123",
            "link": "www.asd.com",
            "federal_case_file_id": self.file1["id"],
            "pages": None,
        }
        self.assertRaises(
            ValidationError,
            self.inst.update,
            id=self.document_reference1["id"],
            params=new_data,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound, self.inst.delete, invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({"id": self.document_reference1["id"],})
        self.inst.delete(id=response["id"])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {"id": self.document_reference1["id"]},
        )

    def test_bulk_delete(self):
        response = self.inst.read({})
        self.assertEqual(2, len(response))

        self.inst.bulk_delete(ptab2_document_id=self.ptab2_document1["id"])

        response = self.inst.read({})
        self.assertEqual(0, len(response))



import copy

from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.File import (
    File,
    FileNotFoundException,
)


class FileResourceTestCase(TestCase):
    def setUp(self):
        super(FileResourceTestCase, self).setUp()
        self.inst = File()
        self.file1 = self.inst.create({
            'file_format': 'txt',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key1',
        })
        self.valid_data = {
            'file_format': 'txt',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key2',
        }

    def test_create_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {
                'file_format': '',
                's3_bucket_name': 'test-bucket',
                's3_key_name': 'key1',
            },
        )

        self.assertRaises(
            ValidationError,
            self.inst.create,
            {
                'file_format': 'txt',
                's3_bucket_name': '',
                's3_key_name': 'key1',
            },
        )

    def test_create_duplicate_s3_bucket_and_key(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {
                'file_format': 'txt',
                's3_bucket_name': 'test-bucket',
                's3_key_name': 'key1',
            },
        )

    def test_create(self):
        data = {
            'file_format': 'pdf',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key2',
            'content_length': None,
        }
        response = self.inst.create(data)
        second_equals_first(data, response)

    def test_create_content_length_not_null(self):
        data = {
            'file_format': 'pdf',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key2',
            'content_length': 123123,
        }
        response = self.inst.create(data)
        second_equals_first(data, response)

    def test_read_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.read,
            {
                'file_format': '',
            },
        )

        self.assertRaises(
            ValidationError,
            self.inst.read,
            {
                's3_bucket_name': '',
            },
        )

    def test_read_all(self):
        # setup
        self.inst.create({
            'file_format': 'txt',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key2',
        })
        response = self.inst.read({})
        self.assertEqual(len(response), 2)

    def test_read_with_params(self):
        # setup
        self.inst.create({
            'file_format': 'txt',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key2',
        })
        response = self.inst.read({'s3_key_name': 'key2'})
        self.assertEqual(len(response), 1)
        self.assertEqual(response[0]['s3_key_name'], 'key2')

        response = self.inst.read({'file_format': 'txt'})
        self.assertEqual(len(response), 2)

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            FileNotFoundException,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.read({'id': self.file1['id']})
        self.assertEqual(len(response), 1)
        response = self.inst.delete(id=self.file1['id'])
        self.assertEqual(response, 'Successfully deleted')
        response = self.inst.read({'id': self.file1['id']})
        self.assertEqual(len(response), 0)

    def test_create_if_does_not_exist(self):
        data = {
            'file_format': 'pdf',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key2',
        }
        response1 = self.inst.create_if_does_not_exist(data)
        second_equals_first(data, response1)

        new_data = {
            'file_format': 'txt',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key2',
        }
        response2 = self.inst.create_if_does_not_exist(new_data)
        self.assertEqual(response2['id'], response1['id'])
        second_equals_first(data, response2)

    def test_update(self):
        new_data = {
            's3_bucket_name': 'some-other-bucket',
            's3_key_name': 'some-other-key',
        }
        resp = self.inst.update(
            id=self.file1['id'],
            params=new_data,
        )
        second_equals_first(
            {
                'id': self.file1['id'],
                's3_bucket_name': new_data['s3_bucket_name'],
                's3_key_name': new_data['s3_key_name'],
            },
            resp,
        )

    def test_update_raises_exception_for_empty_value(self):
        new_data = {
            's3_bucket_name': 'some-other-bucket',
            's3_key_name': '',
        }

        args = []
        kwargs = {
            'id': self.file1['id'],
            'params': new_data,
        }

        self.assertRaises(
            ValidationError,
            self.inst.update,
            *args,
            **kwargs
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                's3_bucket_name': self.valid_data['s3_bucket_name'],
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(1, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(2, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            **self.file1,
            **{'content_length': 123231432},
        }
        resp = self.inst.upsert(data)
        self.assertEqual(self.file1['s3_bucket_name'], resp['s3_bucket_name'])
        self.assertEqual(self.file1['s3_key_name'], resp['s3_key_name'])
        self.assertEqual(data['content_length'], resp['content_length'])
        self.assertEqual(1, len(self.inst.read({})))



from parameterized import parameterized

from src.cs_models.resources.ClaimDependency import ClaimDependency
from test.mixins.ClaimDataMixin import ClaimDataMixin
from test.backendtestcase import TestCase
from test.utils import second_equals_first


class ClaimDependencyResourceTestCase(ClaimDataMixin, TestCase):
    def setUp(self):
        super(ClaimDependencyResourceTestCase, self).setUp()

        self.inst = ClaimDependency()
        self.claim_dependency1 = self.inst.create({
            'claim_id': self.claim1['id'],
            'parent_claim_id': self.claim2['id'],
        })
        self.claim_dependency2 = self.inst.create({
            'claim_id': self.claim3['id'],
            'parent_claim_id': self.claim1['id'],
        })
        self.claim_dependency3 = self.inst.create({
            'claim_id': self.claim3['id'],
            'parent_claim_id': self.claim2['id'],
        })

    def test_create(self):
        data = {
            'claim_id': self.claim1['id'],
            'parent_claim_id': self.claim3['id'],
        }
        resp = self.inst.create(data)
        second_equals_first(
            {
                'claim_id': self.claim1['id'],
                'parent_claim_id': self.claim3['id'],
            },
            resp,
        )

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 3)

    @parameterized.expand([
        ('claim_id', 1),
        ('parent_claim_id', 2),
    ])
    def test_read_w_params(self, query_field, expected_length):
        resp = self.inst.read({
            query_field: self.claim_dependency1[query_field],
        })
        self.assertEqual(expected_length, len(resp))

    def test_delete(self):
        response = self.inst.read({'id': self.claim_dependency1['id']})
        self.assertEqual(len(response), 1)
        response = self.inst.delete(id=self.claim_dependency1['id'])
        self.assertEqual(response, 'Successfully deleted')
        response = self.inst.read({'id': self.claim_dependency1['id']})
        self.assertEqual(len(response), 0)



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Float,
    DECIMAL
)

from ...database import Base


class MergerModel(Base):
    __tablename__ = "mergers"

    id = Column(Integer, primary_key=True)
    target_sec_id = Column(Integer, ForeignKey('companies_sec.id'),
                           nullable=True)
    target_ous_id = Column(Integer, ForeignKey('companies_ous.id'),
                           nullable=True)
    acquirer_sec_id = Column(Integer, ForeignKey('companies_sec.id'),
                             nullable=True)
    acquirer_ous_id = Column(Integer, ForeignKey('companies_ous.id'),
                             nullable=True)
    deal_value = Column(DECIMAL(13, 2), nullable=True)
    type = Column(String(50), nullable=True)
    announcement_date = Column(DateTime, nullable=False)
    offer_price = Column(Float, nullable=True)
    market_price = Column(Float, nullable=True)
    dma_file_id = Column(Integer, ForeignKey('files.id'), nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class NDCResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    spl_id = fields.String(validate=not_blank, required=True)
    product_ndc = fields.String(validate=not_blank, required=True)
    package_ndc = fields.String(validate=not_blank, required=True)
    spl_set_id = fields.String(allow_none=True)
    appl_no = fields.String(allow_none=True)
    appl_type = fields.String(allow_none=True)
    route = fields.String(allow_none=True)
    dosage_form = fields.String(allow_none=True)
    generic_name = fields.String(allow_none=True)
    labeler_name = fields.String(allow_none=True)
    brand_name = fields.String(allow_none=True)
    marketing_category = fields.String(allow_none=True)
    marketing_start_date = fields.DateTime(allow_none=True)
    marketing_end_date = fields.DateTime(allow_none=True)
    labeler_subsidiary_id = fields.Integer(allow_none=True)
    updated_at = fields.DateTime()


class NDCQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    spl_id = fields.String(validate=not_blank)
    product_ndc = fields.String(validate=not_blank)
    package_ndc = fields.String(validate=not_blank)
    route = fields.String(validate=not_blank)


class NDCPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    spl_id = fields.String(validate=not_blank)
    product_ndc = fields.String(validate=not_blank)
    package_ndc = fields.String(validate=not_blank)
    spl_set_id = fields.String(allow_none=True)
    appl_no = fields.String(allow_none=True)
    appl_type = fields.String(allow_none=True)
    route = fields.String(allow_none=True)
    dosage_form = fields.String(allow_none=True)
    generic_name = fields.String(allow_none=True)
    labeler_name = fields.String(allow_none=True)
    brand_name = fields.String(allow_none=True)
    marketing_category = fields.String(allow_none=True)
    marketing_start_date = fields.DateTime(allow_none=True)
    marketing_end_date = fields.DateTime(allow_none=True)
    labeler_subsidiary_id = fields.Integer(allow_none=True)



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
    UniqueConstraint,
)
from datetime import datetime

from ...database import Base


class PatentApplicationModel(Base):
    __tablename__ = 'patent_applications'

    id = Column(Integer, primary_key=True)
    application_number = Column(String(128), nullable=False)
    jurisdiction = Column(String(128), nullable=False)
    abstract_text = Column(Text)
    filed_date = Column(DateTime)
    inventors = Column(Text)
    title = Column(String(500))

    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint('application_number', 'jurisdiction'),)



from sqlalchemy.exc import SQLAlchemyError
from marshmallow import ValidationError

from ...database import db_session
from ...resources.SetIDPharmacologicCross.models import SetIDPharmacologicCrossModel
from .schemas import (
    SetIDPharmacologicCrossPatchSchema,
    SetIDPharmacologicCrossQueryParamsSchema,
    SetIDPharmacologicCrossResourceSchema,
)


schema_resource = SetIDPharmacologicCrossResourceSchema()
schema_params = SetIDPharmacologicCrossQueryParamsSchema()
schema_patch = SetIDPharmacologicCrossPatchSchema()


class SetIDPharmacologicCross(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(SetIDPharmacologicCrossResourceSchema)

        Returns: SetIDPharmacologicCrossResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(SetIDPharmacologicCrossQueryParamsSchema)

        Returns: List<SetIDPharmacologicCrossResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        setid_pharmacologic_cross_query = _build_query(params=data)
        response = schema_resource.dump(setid_pharmacologic_cross_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(SetIDPharmacologicCrossQueryParamsSchema)

        Returns: SetIDPharmacologicCrossResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        setid_pharmacologic_cross_query = _build_query(params=data).one()
        response = schema_resource.dump(setid_pharmacologic_cross_query).data
        return response


def _helper_create(data):
    new_setid_pharmacologic_cross = SetIDPharmacologicCrossModel(**data)
    try:
        db_session.add(new_setid_pharmacologic_cross)
        db_session.commit()
        setid_pharmacologic_query = db_session.query(
            SetIDPharmacologicCrossModel).get(
            new_setid_pharmacologic_cross.spl_set_id)
        response = schema_resource.dump(setid_pharmacologic_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _build_query(params):
    q = db_session.query(SetIDPharmacologicCrossModel)
    if params.get("spl_set_id"):
        q = q.filter_by(spl_set_id=params.get("spl_set_id"))
    if params.get("pharma_set_id"):
        q = q.filter_by(pharma_set_id=params.get("pharma_set_id"))
    return q



from marshmallow import (
    Schema,
    fields,
)


class NCTIdICDCrossResourceSchema(Schema):
    nct_id = fields.String(allow_none=False)
    icd_code = fields.String(allow_none=False)


class NCTIdICDCrossQueryParamsSchema(Schema):
    nct_id = fields.String(allow_none=False)
    icd_code = fields.String(allow_none=False)


class NCTIdICDCrossPatchSchema(Schema):
    nct_id = fields.String(allow_none=False)
    icd_code = fields.String(allow_none=False)



from marshmallow import (
    Schema,
    fields,
)


class SetIDICDCrossResourceSchema(Schema):
    set_id = fields.String(allow_none=False)
    icd_code = fields.String(allow_none=False)


class SetIDICDCrossQueryParamsSchema(Schema):
    set_id = fields.String(allow_none=False)
    icd_code = fields.String(allow_none=False)


class SetIDICDCrossPatchSchema(Schema):
    set_id = fields.String(allow_none=False)
    icd_code = fields.String(allow_none=False)



from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime
from marshmallow import ValidationError

from ...database import db_session
from ...utils.utils import update_model
from .models import (
    AbstractTextModel,
)
from .schemas import (
    AbstractTextResourceSchema,
    AbstractTextQueryParamsSchema,
    AbstractTextPatchSchema,
)


schema_resource = AbstractTextResourceSchema()
schema_params = AbstractTextQueryParamsSchema()
schema_patch = AbstractTextPatchSchema()


class DBException(SQLAlchemyError):
    pass


class AbstractTextNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class AbstractText:
    @staticmethod
    def create(params):
        """
        :param
            Dict
                patent_id: int
                abstract_text: string
        :return:
            newly created object
        :exception:
            ValidationError
            DBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            params: dict
                id
                patent_id
        :return:
            queried object
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        abstract_text_query = _build_query(params=data)
        response = schema_resource.dump(abstract_text_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
            :param
                params: dict
                    id
                    patent_id
            :return:
                queried object
            :exception:
                ValidationError
            """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        abstract_text_query = _build_query(params=data).one()
        response = schema_resource.dump(abstract_text_query).data
        return response

    @staticmethod
    def update(id, params):
        """
        :param
            id: integer: required
            params: dict
                abstract_text
        :return:
            newly updated object
        :exception:
            AbstractTextNotFoundException
            ValidationError
            DBException
        """
        abstract_text_query = db_session.query(
            AbstractTextModel).filter_by(id=id).first()
        if not abstract_text_query:
            raise AbstractTextNotFoundException('Abstract not found!')
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, abstract_text_query)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            AbstractTextNotFoundException
            DBException
        """

        abstract_text_query = db_session.query(
            AbstractTextModel).filter_by(id=id).first()
        if not abstract_text_query:
            raise AbstractTextNotFoundException('Abstract does not exist!')
        try:
            db_session.delete(abstract_text_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise DBException('DB error')


def _helper_create(data):
    new_abstract_text = AbstractTextModel(
        patent_id=data['patent_id'],
        abstract_text=data['abstract_text'],
        updated_at=datetime.utcnow(),
    )
    try:
        db_session.add(new_abstract_text)
        db_session.commit()
        abstract_text_query = db_session.query(
            AbstractTextModel).get(new_abstract_text.id)
        response = schema_resource.dump(abstract_text_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, abstract_text_query):
    data['id'] = abstract_text_query.id
    data['patent_id'] = abstract_text_query.patent_id
    data['updated_at'] = datetime.utcnow()
    try:
        update_model(data, abstract_text_query)
        db_session.commit()
        response = schema_resource.dump(abstract_text_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise DBException('DB error')


def _build_query(params):

    q = db_session.query(AbstractTextModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('patent_id'):
        q = q.filter_by(patent_id=params.get('patent_id'))
    return q



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)
from ...utils.utils import pre_load_date_fields


class ConfigResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    latest_date = fields.DateTime(required=True)
    table_name = fields.String(required=True)
    field_name = fields.String(required=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def convert_string_to_datetime(self, in_data):
        date_fields = ['latest_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m-%d-%Y',
        )
        return in_data


class ConfigQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer()
    table_name = fields.String()
    field_name = fields.String()
    latest_date = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data):
        date_fields = ['date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m-%d-%Y',
        )
        return in_data


class ConfigPatchSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    latest_date = fields.DateTime(allow_none=False)
    table_name = fields.String(allow_none=False)
    field_name = fields.String(allow_none=False)

    @pre_load
    def convert_string_to_datetime(self, in_data):
        date_fields = ['date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m-%d-%Y',
        )
        return in_data



from marshmallow import (
    Schema,
    fields,
)


class SetIDPharmacologicCrossResourceSchema(Schema):
    spl_set_id = fields.String(allow_none=True)
    pharma_set_id = fields.String(allow_none=True)


class SetIDPharmacologicCrossQueryParamsSchema(Schema):
    spl_set_id = fields.String(allow_none=True)
    pharma_set_id = fields.String(allow_none=True)


class SetIDPharmacologicCrossPatchSchema(Schema):
    spl_set_id = fields.String(allow_none=True)
    pharma_set_id = fields.String(allow_none=True)



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)

from ...utils.utils import pre_load_date_fields


class OrangeBookExclusivityResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    appl_no = fields.String(required=True, validate=not_blank)
    appl_type = fields.String(required=True, validate=not_blank)
    product_no = fields.String(required=True, validate=not_blank)
    exclusivity_code = fields.String(required=True, validate=not_blank)
    exclusivity_date = fields.DateTime(allow_none=True)

    @pre_load
    def load_date_fields(self, in_data):
        date_fields = ['exclusivity_date']
        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%b %d, %Y',
        )
        return in_data


class OrangeBookExclusivityQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    appl_type = fields.String(validate=not_blank)
    appl_no = fields.String(validate=not_blank)
    product_no = fields.String(validate=not_blank)
    exclusivity_code = fields.String(validate=not_blank)



from marshmallow import (
    Schema,
    fields,
    validate,
)


class InstitutionProbabilityResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    ptab_trial_num = fields.String(required=True, validate=not_blank)
    is_instituted = fields.Boolean()
    probability_of_institution = fields.Float()
    probability_end_before_trial_end = fields.Float()
    probability_of_all_claims_instituted = fields.Float()
    probability_of_some_claims_instituted = fields.Float()

    updated_at = fields.DateTime(dump_only=True)


class InstitutionProbabilityQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    ptab_trial_num = fields.String(validate=not_blank)



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
    Boolean,
)
from datetime import datetime

from ...database import Base


class CompanySECModel(Base):
    __tablename__ = 'companies_sec'

    id = Column(Integer, primary_key=True)
    cik_str = Column(String(128), nullable=False)
    ticker = Column(String(20), nullable=False, index=True)
    title = Column(String(255), nullable=False, index=True)
    company_url = Column(String(255), nullable=True)
    pipeline_url = Column(String(255), nullable=True)
    ir_url = Column(String(255), nullable=True)
    is_activated = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint('cik_str', 'ticker'),)



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
)

from ...database import Base


class NDCModel(Base):
    __tablename__ = "ndc"

    id = Column(Integer, primary_key=True)
    spl_id = Column(String(128), nullable=False)
    appl_no = Column(String(128), nullable=False)
    appl_type = Column(String(128), nullable=False)
    spl_set_id = Column(String(128), nullable=True)
    product_ndc = Column(String(128), nullable=True, index=True)
    package_ndc = Column(String(128), nullable=True, index=True)
    route = Column(String(128), nullable=True, index=True)
    dosage_form = Column(String(128), nullable=True)
    generic_name = Column(String(128), nullable=True)
    labeler_name = Column(String(128), nullable=True)
    labeler_subsidiary_id = Column(
        Integer, ForeignKey("subsidiaries.id"), nullable=True,
    )
    brand_name = Column(String(128), nullable=True)
    marketing_category = Column(String(128), nullable=True)
    marketing_start_date = Column(DateTime, nullable=True)
    marketing_end_date = Column(DateTime, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from sqlalchemy.exc import SQLAlchemyError
from marshmallow import ValidationError

from cs_models.utils.utils import update_model
from ...database import db_session
from ...resources.DocumentReference.models import (
    DocumentReferenceModel,
)
from .schemas import (
    DocumentReferenceResourceSchema,
    DocumentReferenceQueryParamsSchema,
    DocumentReferencePatchSchema,
)


schema_resource = DocumentReferenceResourceSchema()
schema_params = DocumentReferenceQueryParamsSchema()
schema_patch = DocumentReferencePatchSchema()


class DocumentReference(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(DocumentReferenceResourceSchema)

        Returns: DocumentReferenceResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(DocumentReferenceQueryParamsSchema)

        Returns: List<DocumentReferenceResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        document_ref_query = _build_query(params=data)
        response = schema_resource.dump(
            document_ref_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(DocumentReferenceQueryParamsSchema)

        Returns: DocumentReferenceResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        document_ref_query = _build_query(params=data).one()
        response = schema_resource.dump(document_ref_query).data
        return response

    @staticmethod
    def update(id, params):
        """

        Args:
            id: int
            params: DocumentReferencePatchSchema

        Returns:
            DocumentReferenceResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        document_reference_query = db_session.query(
            DocumentReferenceModel).filter_by(id=id).one()
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, document_reference_query)
        return response

    @staticmethod
    def delete(id):
        """
        Args:
            id: int

        Returns: string

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        document_ref_query = db_session.query(
            DocumentReferenceModel).filter_by(id=id).one()
        try:
            db_session.delete(document_ref_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    @staticmethod
    def bulk_delete(ptab2_document_id):
        try:
            db_session.query(
                DocumentReferenceModel).filter(
                DocumentReferenceModel.ptab2_document_id == ptab2_document_id
            ).delete(synchronize_session=False)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise


def _helper_create(data):
    new_document_reference = DocumentReferenceModel(**data)
    try:
        db_session.add(new_document_reference)
        db_session.commit()
        document_ref_query = db_session.query(
            DocumentReferenceModel).get(
            new_document_reference.id)
        response = schema_resource.dump(document_ref_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, document_reference_query):
    data['id'] = document_reference_query.id
    try:
        update_model(data, document_reference_query)
        db_session.commit()
        response = schema_resource.dump(document_reference_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        DocumentReferenceModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('ptab2_document_id'):
        q = q.filter_by(ptab2_document_id=params.get('ptab2_document_id'))
    if params.get('type'):
        q = q.filter_by(type=params.get('type'))
    if params.get('value'):
        q = q.filter_by(value=params.get('value'))
    if params.get('federal_case_number'):
        q = q.filter_by(federal_case_number=params.get('federal_case_number'))
    if params.get('link'):
        q = q.filter_by(link=params.get('link'))

    return q



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
)

from ...database import Base


class PharmacologicModel(Base):
    __tablename__ = "pharmacologic"

    id = Column(Integer, primary_key=True)
    pharma_set_id = Column(String(128), nullable=False, index=True)
    unii_code = Column(String(128), nullable=True)
    unii_name = Column(String(128), nullable=True)
    nui = Column(String(128), nullable=True, index=True)
    code_system = Column(String(128), nullable=True)
    class_name = Column(String(128), nullable=True, index=True)
    class_type = Column(String(128), nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)
from ...utils.utils import pre_load_date_fields


class ProceedingStageResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    ptab2_proceeding_id = fields.Integer(required=True)
    stage = fields.String(required=True, validate=not_blank)
    is_active = fields.Boolean(required=True)
    filed_date = fields.DateTime(allow_none=True)
    due_date = fields.DateTime(allow_none=True)
    file_id = fields.Integer(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def convert_string_to_datetime(self, in_data):
        date_fields = [
            'filed_date',
            'due_date',
        ]

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m-%d-%Y',
        )
        return in_data



from marshmallow import (
    Schema,
    fields,
    validate,
)


class PatentApplicationResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    application_number = fields.String(required=True)
    jurisdiction = fields.String(required=True)
    # `missing` is used during de-serialization (load) and `default` is
    # used during serialization (dump)
    abstract_text = fields.String(missing=None)
    filed_date = fields.DateTime(missing=None)
    inventors = fields.String(missing=None)
    title = fields.String(missing=None)
    updated_at = fields.DateTime(dump_only=True)


class PatentApplicationQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    application_number = fields.String(validate=not_blank)
    jurisdiction = fields.String(validate=not_blank)



from marshmallow import ValidationError
from freezegun import freeze_time
from test.backendtestcase import TestCase
from sqlalchemy.orm import Session
from src.cs_models.database import engine, operations
from src.cs_models.resources.ICD.models import ICDModel
from src.cs_models.resources.ICD.schemas import ICDResourceSchema


@freeze_time("2020-01-01")
class ICDResourceTestCase(TestCase):
    def setUp(self):
        super(ICDResourceTestCase, self).setUp()
        with operations.session_scope() as session:
            self.icd1 = operations.create(
                session=session,
                model=ICDModel,
                schema=ICDResourceSchema,
                obj={
                    "icd_code": "3B10",
                    "description": "['VIII deficiency', 'Haemophilia A', "
                                   "'anti-factor VIII inhibitor', 'Hereditary factor VIII deficiency']",
                    "class_kind": "category",
                    "is_leaf": False,
                },
            )

        self.valid_data = {
            'icd_code': '3A21',
            'description': "['haemolytic anaemia', 'Paroxysmal nocturnal "
                           "haemoglobinuria', 'Microangiopathic haemolytic anaemia']",
            'class_kind': 'category',
            'is_leaf': False
        }

    def test_create_inside_context_manager(self):
        with operations.session_scope() as session:
            instance = operations.create(
                session=session,
                model=ICDModel,
                schema=ICDResourceSchema,
                obj={
                    "icd_code": "3A50",
                    "description": "['Thalassaemias', 'Haemoglobin H disease']",
                    "class_kind": "category",
                    "is_leaf": False,
                },
            )
            self.assertEqual(
                ICDResourceSchema().dump(instance).data,
                {
                    "icd_code": '3A50',
                    "description": "['Thalassaemias', 'Haemoglobin H disease']",
                    "id": 2,
                    "class_kind": "category",
                    "is_leaf": False,
                    "updated_at": "2020-01-01T00:00:00+00:00",
                },
            )

    def test_create(self):
        session = Session(bind=engine)
        try:
            instance = operations.create(
                session=session,
                model=ICDModel,
                schema=ICDResourceSchema,
                obj={
                    "icd_code": "3A50",
                    "description": "['Thalassaemias', 'Haemoglobin H disease']",
                    "class_kind": "category",
                    "is_leaf": False,
                },
            )
            self.assertEqual(
                ICDResourceSchema().dump(instance).data,
                {
                    "icd_code": '3A50',
                    "description": "['Thalassaemias', 'Haemoglobin H disease']",
                    "id": 2,
                    "class_kind": "category",
                    "is_leaf": False,
                    "updated_at": "2020-01-01T00:00:00+00:00",
                },
            )

            # This commit is not really needed since
            # we commit in `create` but is done for consistency.
            session.commit()
        except Exception:
            session.rollback()
            raise
        finally:
            session.close()

    def test_create_validation_error(self):
        session = Session(bind=engine)
        self.assertRaises(
            ValidationError,
            operations.create,
            session,
            ICDModel,
            ICDResourceSchema,
            {
                "icd_code": "3A50",
                "description": "",
                "class_kind": "category",
                "is_leaf": False,
            },
        )

    def test_update_or_create(self):
        with operations.session_scope() as session:
            instance, created = operations.update_or_create(
                session=session,
                model=ICDModel,
                schema=ICDResourceSchema,
                query_fields=["icd_code"],
                obj={
                    "icd_code": "3A50",
                    "description": "['Thalassaemias', 'Haemoglobin H disease']",
                    "class_kind": "category",
                    "is_leaf": False,
                },
            )
            self.assertEqual(
                ICDResourceSchema().dump(instance).data,
                {
                    "icd_code": "3A50",
                    "id": 2,
                    "description": "['Thalassaemias', 'Haemoglobin H disease']",
                    "class_kind": "category",
                    "is_leaf": False,
                    "updated_at": "2020-01-01T00:00:00+00:00",
                },
            )
            self.assertTrue(created)

            instance, created = operations.update_or_create(
                session=session,
                model=ICDModel,
                schema=ICDResourceSchema,
                query_fields=["icd_code"],
                obj={
                    "icd_code": "3A50",
                    "description": "['Hemophilia']",
                    "class_kind": "category",
                    "is_leaf": False,
                },
            )
            self.assertEqual(
                ICDResourceSchema().dump(instance).data,
                {
                    "icd_code": "3A50",
                    "id": 2,
                    "description": "['Hemophilia']",
                    "class_kind": "category",
                    "is_leaf": False,
                    "updated_at": "2020-01-01T00:00:00+00:00",
                },
            )
            self.assertFalse(created)

    def test_bulk_save(self):
        """Verifies that new objects are created in bulk."""
        with operations.session_scope() as session:
            operations.bulk_save(
                session=session,
                instances=[
                    ICDModel(**obj)
                    for obj in [
                        {
                            "icd_code": "3A50",
                            "description": "['Hemophilia']",
                            "class_kind": "category",
                            "is_leaf": False,
                        },
                        {
                            "icd_code": "3A60",
                            "description": "['Colitis']",
                            "class_kind": "category",
                            "is_leaf": False,
                        },
                    ]
                ],
            )
            self.assertEqual(
                ICDResourceSchema()
                .dump(session.query(ICDModel).all(), many=True)
                .data,
                [
                    {
                        "id": 1,
                        "icd_code": "3B10",
                        "description": "['VIII deficiency', 'Haemophilia A', "
                                       "'anti-factor VIII inhibitor', 'Hereditary factor VIII deficiency']",
                        "class_kind": "category",
                        "is_leaf": False,
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                    {
                        "id": 2,
                        "icd_code": "3A50",
                        "description": "['Hemophilia']",
                        "class_kind": "category",
                        "is_leaf": False,
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                    {
                        "id": 3,
                        "icd_code": "3A60",
                        "description": "['Colitis']",
                        "class_kind": "category",
                        "is_leaf": False,
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                ],
            )

    def test_bulk_save_updates_and_creates(self):
        with operations.session_scope() as session:
            instance1 = operations.create(
                session=session,
                model=ICDModel,
                schema=ICDResourceSchema,
                obj={
                    "icd_code": "3A50",
                    "description": "['Hemophilia']",
                    "class_kind": "category",
                    "is_leaf": False,
                },
            )
            instance2 = operations.create(
                session=session,
                model=ICDModel,
                schema=ICDResourceSchema,
                obj={
                    "icd_code": "3A60",
                    "description": "['Colitis']",
                    "class_kind": "category",
                    "is_leaf": False,
                },
            )
            instance1.description = "['Hemophilia A']"
            instance2.class_kind = "block"

            operations.bulk_save(
                session=session,
                instances=[
                    instance1,
                    instance2,
                    ICDModel(
                        **{
                            "icd_code": "3A70",
                            "description": "['Cancer']",
                            "class_kind": "category",
                            "is_leaf": False,
                        }
                    ),
                    ICDModel(
                        **{
                            "icd_code": "3A80",
                            "description": "['Alzheimer']",
                            "class_kind": "category",
                            "is_leaf": False,
                        }
                    ),
                ],
            )

            self.assertEqual(
                [
                    {
                        "id": 1,
                        "icd_code": "3B10",
                        "description": "['VIII deficiency', 'Haemophilia A', "
                                       "'anti-factor VIII inhibitor', 'Hereditary factor VIII deficiency']",
                        "class_kind": "category",
                        "is_leaf": False,
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                    {
                        "id": 2,
                        "icd_code": "3A50",
                        "description": "['Hemophilia A']",
                        "class_kind": "category",
                        "is_leaf": False,
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    }
                    ,
                    {
                        "id": 3,
                        "icd_code": "3A60",
                        "description": "['Colitis']",
                        "class_kind": "block",
                        "is_leaf": False,
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                    {
                        "id": 4,
                        "icd_code": "3A70",
                        "description": "['Cancer']",
                        "class_kind": "category",
                        "is_leaf": False,
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                    {
                        "id": 5,
                        "icd_code": "3A80",
                        "description": "['Alzheimer']",
                        "class_kind": "category",
                        "is_leaf": False,
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                ],
                ICDResourceSchema()
                .dump(session.query(ICDModel).all(), many=True)
                .data,
            )



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
    Text,
)

from ...database import Base


class CompanyConditionStageModel(Base):
    __tablename__ = "company_condition_stage"

    id = Column(Integer, primary_key=True)
    src_condition_id = Column(String(128), nullable=False, index=True)
    stage = Column(Text)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    __table_args__ = (UniqueConstraint("src_condition_id"),)



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError

from ...database import db_session
from ...resources.TrialStat.models import (
    TrialStatModel,
)
from .schemas import (
    TrialStatResourceSchema,
    TrialStatQueryParamsSchema,
)
from ...utils.utils import update_model


schema_resource = TrialStatResourceSchema()
schema_params = TrialStatQueryParamsSchema()


class TrialStat(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(TrialStatResourceSchema)

        Returns: TrialStatResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(TrialStatQueryParamsSchema)

        Returns: List<TrialStatResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        trial_stat_query = _build_query(params=data)
        response = schema_resource.dump(
            trial_stat_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(TrialStatQueryParamsSchema)

        Returns: TrialStatResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        trial_stat_query = _build_query(params=data).one()
        response = schema_resource.dump(trial_stat_query).data
        return response

    @staticmethod
    def upsert(params):
        """
        Args:
            params: TrialStatResourceSchema

        Returns:
            TrialStatResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        try:
            query_params = {
                'stat_type': params['stat_type'],
                'name': params['name'],
            }
            trial_stat_query = _build_query(query_params).one()
            response = _helper_update(data, trial_stat_query)
        except NoResultFound:
            response = _helper_create(data)
        return response


def _helper_create(data):
    new_trial_stat = TrialStatModel(**data)
    try:
        db_session.add(new_trial_stat)
        db_session.commit()
        trial_stat_query = db_session.query(TrialStatModel).get(
            new_trial_stat.id)
        response = schema_resource.dump(trial_stat_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, trial_stat_query):
    data['id'] = trial_stat_query.id
    data['stat_type'] = trial_stat_query.stat_type
    data['name'] = trial_stat_query.name
    try:
        update_model(data, trial_stat_query)
        db_session.commit()
        response = schema_resource.dump(trial_stat_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        TrialStatModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('stat_type'):
        q = q.filter_by(
            stat_type=params.get('stat_type'))
    if params.get('name'):
        q = q.filter_by(
            name=params.get('name'))
    return q



from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.SetIDPharmacologicCross import SetIDPharmacologicCross


class SetIDPharmacologicCrossResourceTestCase(TestCase):
    def setUp(self):
        super(SetIDPharmacologicCrossResourceTestCase, self).setUp()
        self.inst = SetIDPharmacologicCross()

        self.setidpharmacross1 = self.inst.create({
            "spl_set_id": '000155a8-709c-44e5-a75f-cd890f3a7caf',
            "pharma_set_id": 'c29a4f1b-49e6-46a6-aefe-820474b74d5f',

        })

        self.setidpharmacross2 = self.inst.create({
            "spl_set_id": '001dcaf3-5838-4fb0-a4b8-d65ae2eb009e',
            "pharma_set_id": '60da9c53-1225-442f-bbe1-3e0b1175c6e5',

        })

        self.valid_data = {
            "spl_set_id": '0045ec13-789a-40a1-ac1f-8c29049cfd13',
            "pharma_set_id": '49165386-3fd3-40b9-b5b8-89eb6f691830',
        }

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        expected_data = {
            **self.valid_data,
        }
        second_equals_first(expected_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

    @parameterized.expand([
        ('spl_set_id', 'setidpharmacross1', 'spl_set_id', 1),
        ('pharma_set_id', 'setidpharmacross1', 'pharma_set_id', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('spl_set_id', '999', NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('spl_set_id',),
        ('pharma_set_id',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.setidpharmacross1[field_name],
        })
        second_equals_first(
            self.setidpharmacross1,
            resp,
        )



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
    ForeignKey,
)
from datetime import datetime

from ...database import Base


class NewswireModel(Base):
    __tablename__ = 'newswires'

    id = Column(Integer, primary_key=True)
    date = Column(DateTime)
    tzinfo = Column(String(10))
    source = Column(String(128))
    headline = Column(Text)
    drugs = Column(Text)
    conditions = Column(Text)
    filtered_drugs = Column(Text)
    filtered_conditions = Column(Text)
    subsidiary_id = Column(
        Integer,
        ForeignKey('subsidiaries.id'),
        nullable=True,
    )
    news_file_id = Column(Integer, ForeignKey("files.id"))
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime
from marshmallow import ValidationError

from ...database import db_session
from ...utils.utils import update_model
from ...resources.PriorArt.models import PriorArtModel
from .schemas import (
    PriorArtResourceSchema,
    PriorArtQueryParamsSchema,
    PriorArtPatchSchema,
)


schema_resource = PriorArtResourceSchema()
schema_params = PriorArtQueryParamsSchema()
schema_patch = PriorArtPatchSchema()


class PriorArtDBException(SQLAlchemyError):
    pass


class PriorArtNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class PriorArt:
    @staticmethod
    def create(params):
        """
        :param
            PriorArtResourceSchema
        :return:
            PriorArtResourceSchema
        :exception:
            ValidationError
            PriorArtDBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            PriorArtQueryParamsSchema
        :return:
            queried object
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        prior_art_query = _build_query(params=data)
        response = schema_resource.dump(prior_art_query, many=True).data
        return response

    @staticmethod
    def update(id, params):
        """
        :param
            id: integer: required
            params: dict
                prior_art
                prior_art_detail
        :return:
            PriorArtResourceSchema
        :exception:
            PriorArtNotFoundException
            ValidationError
            PriorArtDBException
        """
        prior_art_query = db_session.query(
            PriorArtModel).filter_by(id=id).first()
        if not prior_art_query:
            raise PriorArtNotFoundException('Prior art not found!')
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, prior_art_query)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            PriorArtNotFoundException
            PriorArtDBException
        """

        prior_art_query = db_session.query(
            PriorArtModel).filter_by(id=id).first()
        if not prior_art_query:
            raise PriorArtNotFoundException('Prior art does not exist!')
        try:
            db_session.delete(prior_art_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise PriorArtDBException('DB error')

    def upsert(self, params):
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        prior_art_query = db_session.query(
            PriorArtModel).filter_by(
            ptab2_document_id=params['ptab2_document_id'],
            tag=params['tag'],
        ).first()

        if not prior_art_query:
            response = _helper_create(data)
        else:
            response = _helper_update(data, prior_art_query)
        return response

    @staticmethod
    def bulk_delete(ptab2_document_id):
        try:
            db_session.query(PriorArtModel).filter(
                PriorArtModel.ptab2_document_id == ptab2_document_id
            ).delete(synchronize_session=False)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    def bulk_update(self, ptab2_document_id, patch_data):
        prior_arts = self.read({'ptab2_document_id': ptab2_document_id})
        allowed_prior_art_ids = [pa['id'] for pa in prior_arts]
        update_data = {}
        for prior_art_id in patch_data:
            if int(prior_art_id) in allowed_prior_art_ids:
                update_data[int(prior_art_id)] = patch_data[prior_art_id]

        for prior_art_id in update_data:
            self.update(
                id=prior_art_id,
                params={'prior_art_detail': update_data[prior_art_id]},
            )
        return self.read({'ptab2_document_id': ptab2_document_id})


def _helper_create(data):
    new_prior_art = PriorArtModel(
        ptab2_document_id=data['ptab2_document_id'],
        tag=data['tag'],
        title=data['title'],
        exhibit=data['exhibit'],
        updated_at=datetime.utcnow(),
    )
    try:
        db_session.add(new_prior_art)
        db_session.commit()
        prior_art_query = db_session.query(
            PriorArtModel).get(new_prior_art.id)
        response = schema_resource.dump(prior_art_query).data
        db_session.close()
        return response
    except SQLAlchemyError as e:
        db_session.rollback()
        db_session.close()
        raise PriorArtDBException('DB error {}'.format(str(e)))


def _helper_update(data, prior_art_query):
    data['id'] = prior_art_query.id
    data['ptab2_document_id'] = prior_art_query.ptab2_document_id
    data['tag'] = prior_art_query.tag
    data['updated_at'] = datetime.utcnow()
    try:
        update_model(data, prior_art_query)
        db_session.commit()
        response = schema_resource.dump(prior_art_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise PriorArtDBException('DB error')


def _build_query(params):
    q = db_session.query(
        PriorArtModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('ptab2_document_id'):
        q = q.filter_by(ptab2_document_id=params.get('ptab2_document_id'))
    if params.get('tag'):
        q = q.filter_by(tag=params.get('tag'))
    return q



from marshmallow import ValidationError
from sqlalchemy.orm.exc import (
    NoResultFound,
    MultipleResultsFound,
)

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.PatentIdentifierSynonym import (
    PatentIdentifierSynonym,
)
from src.cs_models.resources.Patent import Patent


class PatentIdentifierSynonymResourceTestCase(TestCase):
    def setUp(self):
        super(PatentIdentifierSynonymResourceTestCase, self).setUp()
        self.inst_patent = Patent()
        self.inst = PatentIdentifierSynonym()
        self.patent1 = self.inst_patent.create({
            'patent_number': '1000',
            'app_grp_art_number': '5000',
            'jurisdiction': 'USPAT',
        })
        self.patent2 = self.inst_patent.create({
            'patent_number': '2000',
            'app_grp_art_number': '5000',
            'jurisdiction': 'USPAT',
        })
        self.patent_identifier_synonym1 = self.inst.create({
            'patent_id': self.patent1['id'],
            'synonym': 'some synonym',
        })

    def test_create_validation_error_missing_fields(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {},
        )

    def test_create_validation_error_blank_synonym(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {
                'patent_id': 123,
                'synonym': '',
            },
        )

    def test_create_duplicate_patent_id(self):
        self.assertRaises(
            Exception,
            self.inst.create,
            {
                'patent_id': self.patent_identifier_synonym1['patent_id'],
                'synonym': self.patent_identifier_synonym1['synonym'],
            }
        )

    def test_create(self):
        new_data = {
            'patent_id': self.patent2['id'],
            'synonym': 'some synonym',
        }
        response = self.inst.create(new_data)
        second_equals_first(new_data, response)

    def test_read_validation_error_blank_synonym(self):
        self.assertRaises(
            ValidationError,
            self.inst.read,
            {'synonym': ''},
        )

    def test_read_all(self):
        response = self.inst.read({})
        self.assertEqual(len(response), 1)

    def test_read_w_params(self):
        # setup
        self.inst.create({
            'patent_id': self.patent1['id'],
            'synonym': 'some synonym2',
        })
        self.inst.create({
            'patent_id': self.patent2['id'],
            'synonym': self.patent_identifier_synonym1['synonym'],
        })

        response = self.inst.read({})
        self.assertEqual(len(response), 3)

        response = self.inst.read({
            'synonym': self.patent_identifier_synonym1['synonym']
        })
        self.assertEqual(len(response), 2)

    def test_one_raises_MultipleResultsFound(self):
        # setup
        self.inst.create({
            'patent_id': self.patent2['id'],
            'synonym': self.patent_identifier_synonym1['synonym'],
        })

        self.assertRaises(
            MultipleResultsFound,
            self.inst.one,
            {'synonym': self.patent_identifier_synonym1['synonym']},
        )

    def test_one_raises_NoResultFound(self):
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'synonym': '123123'},
        )



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)
from ...utils.utils import pre_load_date_fields


class PacerCaseResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    case_no = fields.String(required=True, validate=not_blank)
    court_id = fields.String(required=True, validate=not_blank)
    pacer_case_external_id = fields.String(required=True, validate=not_blank)
    cause = fields.String()
    county = fields.String()
    defendant = fields.String()
    disposition = fields.String()
    filed_date = fields.DateTime(allow_none=True)
    flags = fields.String()
    jurisdiction = fields.String()
    lead_case = fields.String()
    nature_of_suit = fields.String()
    office = fields.String()
    plaintiff = fields.String()
    related_case = fields.String()
    terminated_date = fields.DateTime(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def convert_string_to_datetime(self, in_data):
        date_fields = ['filed_date', 'terminated_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m/%d/%Y',
        )
        return in_data


class PacerCaseQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    ids = fields.List(fields.Integer())
    case_no = fields.String()
    court_id = fields.String()
    pacer_case_external_id = fields.String()

    @pre_load
    def convert_string_to_datetime(self, in_data):
        date_fields = ['filed_date', 'terminated_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m/%d/%Y',
        )
        return in_data


class PacerCasePatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    case_no = fields.String(validate=not_blank)
    court_id = fields.String(required=True, validate=not_blank)
    pacer_case_external_id = fields.String(required=True, validate=not_blank)
    cause = fields.String()
    county = fields.String()
    defendant = fields.String()
    disposition = fields.String()
    filed_date = fields.DateTime()
    flags = fields.String()
    jurisdiction = fields.String()
    lead_case = fields.String()
    nature_of_suit = fields.String()
    office = fields.String()
    plaintiff = fields.String()
    related_case = fields.String()
    terminated_date = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data):
        date_fields = ['filed_date', 'terminated_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m/%d/%Y',
        )
        return in_data



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)

from ...utils.utils import pre_load_date_fields


class PTAB2ProceedingResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    accorded_filing_date = fields.DateTime(allow_none=True)
    max_document_date = fields.DateTime(allow_none=True)
    decision_date = fields.DateTime(allow_none=True)
    institution_decision_date = fields.DateTime(allow_none=True)
    petitioner_application_number_text = fields.String(allow_none=True)
    petitioner_counsel_name = fields.String(allow_none=True)
    petitioner_grant_date = fields.DateTime(allow_none=True)
    petitioner_group_art_unit_number = fields.String(allow_none=True)
    petitioner_inventor_name = fields.String(allow_none=True)
    petitioner_party_name = fields.String(allow_none=True)
    petitioner_patent_number = fields.String(allow_none=True)
    petitioner_patent_owner_name = fields.String(allow_none=True)
    petitioner_technology_center_number = fields.String(allow_none=True)
    proceeding_filing_date = fields.DateTime(allow_none=True)
    proceeding_last_modified_date = fields.DateTime(allow_none=True)
    proceeding_number = fields.String(required=True)
    proceeding_status_category = fields.String(allow_none=True)
    proceeding_type_category = fields.String(allow_none=True)
    respondent_application_number_text = fields.String(allow_none=True)
    respondent_counsel_name = fields.String(allow_none=True)
    respondent_grant_date = fields.DateTime(allow_none=True)
    respondent_group_art_unit_number = fields.String(allow_none=True)
    respondent_inventor_name = fields.String(allow_none=True)
    respondent_party_name = fields.String(allow_none=True)
    respondent_patent_number = fields.String(allow_none=True)
    respondent_patent_owner_name = fields.String(allow_none=True)
    respondent_technology_center_number = fields.String(allow_none=True)
    subproceeding_type_category = fields.String(allow_none=True)
    proceeding_status = fields.String(allow_none=True)
    appealed = fields.Boolean(allow_none=True)
    respondent_subsidiary_id = fields.Integer(allow_none=True)
    petitioner_subsidiary_id = fields.Integer(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def convert_string_to_datetime(self, in_data):
        date_fields = [
            'accorded_filing_date',
            'decision_date',
            'institution_decision_date',
            'petitioner_grant_date',
            'proceeding_filing_date',
            'proceeding_last_modified_date',
            'respondent_grant_date',
            'max_document_date',
        ]

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%m-%d-%Y',
        )
        return in_data


class PTAB2ProceedingQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    proceeding_number = fields.String()



from marshmallow import (
    Schema,
    fields,
    validate,
)


class CompanyResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    applicant_full_name = fields.String(required=True, validate=not_blank)
    exchange = fields.String()
    parent_company = fields.String()
    ticker = fields.String()
    company_sec_id = fields.Integer(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)


class CompanyQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    applicant_full_name = fields.String(validate=not_blank)
    parent_company = fields.String(validate=not_blank)
    ticker = fields.String(validate=not_blank)



from sqlalchemy.exc import SQLAlchemyError

from ...database import db_session
from ...resources.PriorArtRefRelation.models import (
    PriorArtRefRelationModel,
)
from .schemas import (
    PriorArtRefRelationResourceSchema,
    PriorArtRefRelationQueryParamsSchema,
)
from marshmallow import ValidationError


schema_resource = PriorArtRefRelationResourceSchema()
schema_params = PriorArtRefRelationQueryParamsSchema()


class PriorArtRefRelation:
    @staticmethod
    def create(params):
        """
        Args:
            params: Dict (PriorArtRefRelationResourceSchema)

        Returns:
            PriorArtRefRelationResourceSchema

        Raises:
            ValidationError
            DBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: PriorArtRefRelationQueryParamsSchema

        Returns:
            List<PriorArtRefRelationResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        pa_ref_relation_query = _build_query(params=data)
        response = schema_resource.dump(pa_ref_relation_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: PriorArtRefRelationQueryParamsSchema

        Returns:
            PriorArtRefRelationResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        pa_ref_relation_query = _build_query(params=data).one()
        response = schema_resource.dump(pa_ref_relation_query).data
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """

        pa_ref_relation_query = db_session.query(
            PriorArtRefRelationModel).filter_by(
            id=id).one()
        try:
            db_session.delete(pa_ref_relation_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise


def _helper_create(data):
    new_pa_ref_relation = PriorArtRefRelationModel(**data)
    try:
        db_session.add(new_pa_ref_relation)
        db_session.commit()
        pa_ref_relation_query = db_session.query(
            PriorArtRefRelationModel).get(
            new_pa_ref_relation.id)
        response = schema_resource.dump(pa_ref_relation_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _build_query(params):
    q = db_session.query(
        PriorArtRefRelationModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('prior_art_id'):
        q = q.filter_by(prior_art_id=params.get('prior_art_id'))
    if params.get('cross_ref_id'):
        q = q.filter_by(cross_ref_id=params.get('cross_ref_id'))
    if params.get('patent_id'):
        q = q.filter_by(patent_id=params.get('patent_id'))
    if params.get('patent_application_id'):
        q = q.filter_by(
            patent_application_id=params.get('patent_application_id'))
    return q



from marshmallow import (
    Schema,
    fields,
    validate,
)


class NCTIdSubsidiaryResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    nct_id = fields.String(validate=not_blank, required=True)
    subsidiary_id = fields.Integer(required=True)
    lead_or_collaborator = fields.String()
    updated_at = fields.DateTime()



from copy import copy
from test.backendtestcase import TestCase
from test.utils import second_equals_first

from marshmallow import ValidationError
from parameterized import parameterized
from sqlalchemy.exc import IntegrityError

from src.cs_models.resources.Subsidiary import Subsidiary
from src.cs_models.resources.CompanySEC import CompanySEC
from src.cs_models.resources.CompanyOUS import CompanyOUS
from src.cs_models.resources.OrangeBookProduct import (
    OrangeBookProduct,
    OrangeBookProductNotFoundException,
)


class OrangeBookProductResourceTestCase(TestCase):
    def setUp(self):
        super(OrangeBookProductResourceTestCase, self).setUp()
        self.inst_subsidiary = Subsidiary()
        self.inst_sec_company = CompanySEC()
        self.inst_ous_company = CompanyOUS()
        self.inst = OrangeBookProduct()

        self.company_sec1 = self.inst_sec_company.create(
            {
                "cik_str": "200406",
                "ticker": "JNJ",
                "title": "JOHNSON & JOHNSON",
            }
        )

        self.subsidiary1 = self.inst_subsidiary.create(
            {
                "name": "Janssen Scientific Affairs, LLC",
                "company_sec_id": self.company_sec1["id"],
                "company_ous_id": None,
            }
        )

        self.ob_product1 = self.inst.create(
            {
                "appl_no": "78907",
                "appl_type": "A",
                "applicant": "SPECGX LLC",
                "applicant_full_name": "SPECGX LLC",
                "applicant_subsidiary_id": None,
                "approval_date": "Oct 30, 2009",
                "dosage_form": "TROCHE/LOZENGE",
                "ingredient": "FENTANYL CITRATE",
                "product_no": "6",
                "rld": "No",
                "rs": "No",
                "route_of_administration": "TRANSMUCOSAL",
                "strength": "EQ 1.6MG BASE",
                "te_code": "AB",
                "trade_name": "FENTANYL CITRATE",
                "type": "RX",
                "drug_set_id": None,
            }
        )
        self.valid_data = {
            "appl_no": "12312",
            "appl_type": "A",
            "applicant": "Something",
            "applicant_full_name": "some llc",
            "applicant_subsidiary_id": self.subsidiary1["id"],
            "approval_date": "Oct 30, 2009",
            "dosage_form": "TROCHE/LOZENGE",
            "ingredient": "FENTANYL CITRATE",
            "product_no": "6",
            "rld": "No",
            "rs": "No",
            "route_of_administration": "TRANSMUCOSAL",
            "strength": "EQ 1.6MG BASE",
            "te_code": "AB",
            "trade_name": "FENTANYL CITRATE",
            "type": "RX",
            "drug_set_id": None,
        }

    def test_create_validation_error_missing_fields(self):
        self.assertRaises(ValidationError, self.inst.create, {})

    def test_invalid_applicant_subsidiary_id_raises_exception(self):
        data = copy(self.valid_data)
        invalid_subsidiary_id = 3324
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {**data, **{"applicant_subsidiary_id": invalid_subsidiary_id}},
        )

    def test_create(self):
        data = copy(self.valid_data)
        response = self.inst.create(data)
        data.pop("approval_date")
        second_equals_first(data, response)

    def test_read_validation_error_blank_appl_no(self):
        self.assertRaises(
            ValidationError, self.inst.read, {"appl_no": ""},
        )

    def test_read_all(self):
        response = self.inst.read({})
        self.assertEqual(len(response), 1)

    @parameterized.expand(
        [("appl_no", 1), ("product_no", 2), ("trade_name", 2),]
    )
    def test_read_w_params(self, attr, expected_length):
        # setup
        new_ob_product = self.inst.create(
            {
                "appl_no": "123123",
                "appl_type": "A",
                "applicant": "SPECGX LLC",
                "applicant_full_name": "SPECGX LLC",
                "approval_date": "Oct 30, 2009",
                "dosage_form": "TROCHE/LOZENGE",
                "ingredient": "FENTANYL CITRATE",
                "product_no": self.ob_product1["product_no"],
                "rld": "No",
                "rs": "No",
                "route_of_administration": "TRANSMUCOSAL",
                "strength": "EQ 1.6MG BASE",
                "te_code": "AB",
                "trade_name": "FENTANYL",
                "type": "RX",
            }
        )
        self.inst.create(
            {
                "appl_no": "324234",
                "appl_type": "A",
                "applicant": "SPECGX LLC",
                "applicant_full_name": "SPECGX LLC",
                "approval_date": "Oct 30, 2009",
                "dosage_form": "TROCHE/LOZENGE",
                "ingredient": "FENTANYL CITRATE",
                "product_no": "123324234",
                "rld": "No",
                "rs": "No",
                "route_of_administration": "TRANSMUCOSAL",
                "strength": "EQ 1.6MG BASE",
                "te_code": "AB",
                "trade_name": "Copaxone",
                "type": "RX",
            }
        )

        response = self.inst.read({})
        self.assertEqual(3, len(response))

        response = self.inst.read({attr: new_ob_product[attr]})
        self.assertEqual(expected_length, len(response))

    def test_update_not_found(self):
        invalid_id = 99999
        args = []
        kwargs = {
            "id": invalid_id,
            "params": {
                "appl_no": "123123",
                "appl_type": "A",
                "applicant": "SPECGX LLC",
                "applicant_full_name": "SPECGX LLC",
                "approval_date": "Oct 30, 2009",
                "dosage_form": "TROCHE/LOZENGE",
                "ingredient": "FENTANYL CITRATE",
                "product_no": self.ob_product1["product_no"],
                "rld": "No",
                "rs": "No",
                "route_of_administration": "TRANSMUCOSAL",
                "strength": "EQ 1.6MG BASE",
                "te_code": "AB",
                "trade_name": "FENTANYL CITRATE",
                "type": "RX",
            },
        }
        self.assertRaises(
            OrangeBookProductNotFoundException,
            self.inst.update,
            *args,
            **kwargs,
        )

    @parameterized.expand(
        ["appl_no", "product_no", "drug_set_id"]
    )
    def test_update(self, field_name):
        new_data = {
            "appl_no": "2000",
            "appl_type": "ABBB",
            "applicant": "SPECGX LLC",
            "applicant_full_name": "SPECGX LLC",
            "approval_date": "Oct 30, 2009",
            "dosage_form": "TROCHE/LOZENGE",
            "ingredient": "FENTANYL CITRATE",
            "product_no": "123123123",
            "rld": "No",
            "rs": "No",
            "route_of_administration": "TRANSMUCOSAL",
            "strength": "EQ 1.6MG BASE",
            "te_code": "AB",
            "trade_name": "FENTANYL CITRATE",
            "type": "RX",
            "drug_set_id": "43a44fd3-4ba6-40c9-929f-8600f8da4222",
        }
        response = self.inst.update(
            id=self.ob_product1["id"], params=new_data,
        )
        self.assertEqual(response["id"], self.ob_product1["id"])
        self.assertEqual(
            response[field_name], new_data[field_name],
        )
        self.assertEqual(
            response["appl_type"], new_data["appl_type"],
        )



import copy
from marshmallow import ValidationError
from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.Subsidiary import Subsidiary
from src.cs_models.resources.CompanySEC import CompanySEC
from src.cs_models.resources.CompanyOUS import CompanyOUS
from src.cs_models.resources.NDC import NDC


class NDCResourceTestCase(TestCase):
    def setUp(self):
        super(NDCResourceTestCase, self).setUp()
        self.inst_subsidiary = Subsidiary()
        self.inst_sec_company = CompanySEC()
        self.inst_ous_company = CompanyOUS()
        self.inst = NDC()

        self.company_sec1 = self.inst_sec_company.create(
            {
                "cik_str": "200406",
                "ticker": "TEVA",
                "title": "Teva Pharmaceuticals",
            }
        )

        self.company_sec2 = self.inst_sec_company.create(
            {
                "cik_str": "332434",
                "ticker": "RHBBY",
                "title": "ROCHE",
            }
        )

        self.subsidiary1 = self.inst_subsidiary.create(
            {
                "name": "Teva Pharma USA",
                "company_sec_id": self.company_sec1["id"],
                "company_ous_id": None,
            }
        )

        self.subsidiary2 = self.inst_subsidiary.create(
            {
                "name": "Roche",
                "company_sec_id": self.company_sec2["id"],
                "company_ous_id": None,
            }
        )

        self.ndc1 = self.inst.create({
            "spl_id": "10a84bba-6ac6-4a54-9c2b-55d39a232d59",
            'appl_no': "761083",
            'appl_type': 'B',
            'spl_set_id': None,
            "product_ndc": "50242-920",
            "package_ndc": "50242-920-01",
            "route": "Subcutaneous",
            "dosage_form": "Injection, Solution",
            "generic_name": "emicizumab",
            "labeler_name": "Genentech, Inc.",
            "brand_name": "Hemlibra",
            "marketing_category": "BLA",
            "labeler_subsidiary_id": self.subsidiary2["id"],
        })

        self.ndc2 = self.inst.create({
            "spl_id": "15ecddea-ee95-45d1-a5cc-ba1e530b3847",
            "appl_no": "020622",
            'appl_type': 'N',
            'spl_set_id': None,
            "product_ndc": "68546-317",
            "package_ndc": "68546-317-01",
            "route": "Oral",
            "dosage_form": "Tablet",
            "generic_name": "Glatiramer Acetate",
            "labeler_name": "Teva Neuroscience, Inc.",
            "brand_name_base": "Copaxone",
            "marketing_category": "NDA",
            "labeler_subsidiary_id": self.subsidiary1["id"],
        })

        self.valid_data = {
            "spl_id": "9f6112fb-78ef-43cf-8ae3-36370eb45468",
            "appl_no": "021658",
            'appl_type': 'N',
            'spl_set_id': "9f6112fb-78ef-43cf-8ae3-36370eb45468",
            "product_ndc": "70515-711",
            "package_ndc": "70515-711-01",
            "route": "Oral",
            "dosage_form": "Tablet",
            "generic_name": "ciclesonide",
            "labeler_name": "Covis Pharma",
            "brand_name": "Alvesco",
            "marketing_category": "NDA",
            "labeler_subsidiary_id": self.subsidiary1["id"],
        }

    @parameterized.expand([
        ('spl_id',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        expected_data = {
            **self.valid_data,
        }
        second_equals_first(expected_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

    @parameterized.expand([
        ('id', 'ndc1', 'id', 1),
        ('spl_id', 'ndc1', 'spl_id', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('spl_id',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.ndc1[field_name],
        })
        second_equals_first(
            self.ndc1,
            resp,
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'spl_id': self.valid_data['spl_id'],
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(2, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            **self.valid_data,
            **{'spl_id': self.ndc1['spl_id'],
               'appl_no': self.ndc1['appl_no'],
               'spl_set_id': self.ndc1['spl_set_id'],
               'product_ndc': self.ndc1['product_ndc'],
               'generic_name': self.ndc1['generic_name'],
               "route": self.ndc1["route"],
               "dosage_form": self.ndc1["dosage_form"],
               'labeler_name': self.ndc1['labeler_name'],
               'brand_name': self.ndc1['brand_name'],
               'marketing_category': self.ndc1['marketing_category'],
               "labeler_subsidiary_id": self.subsidiary2["id"],
               },
        }
        resp = self.inst.upsert(data)
        expected_data = {
            **data,
        }
        second_equals_first(
            expected_data,
            resp,
        )
        self.assertEqual(2, len(self.inst.read({})))



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
    ValidationError,
)


class SubsidiaryResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    name = fields.String(required=True, validate=not_blank)
    company_sec_id = fields.Integer(allow_none=True)
    company_ous_id = fields.Integer(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def pre_load(self, in_data):
        if self._get_number_of_company_fields(in_data) > 1:
            raise ValidationError('Provide either company_sec_id or '
                                  'company_ous_id, not both')

    def _get_number_of_company_fields(self, in_data):
        result = 0
        if 'company_sec_id' in in_data:
            if in_data['company_sec_id'] is not None:
                result += 1
        if 'company_ous_id' in in_data:
            if in_data['company_ous_id'] is not None:
                result += 1
        return result


class SubsidiaryQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    name = fields.String(validate=not_blank)



import copy
from marshmallow import ValidationError
from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.DrugIndication import DrugIndication


class DrugIndicationResourceTestCase(TestCase):
    def setUp(self):
        super(DrugIndicationResourceTestCase, self).setUp()
        self.inst = DrugIndication()

        self.indication1 = self.inst.create({
            'set_id': '3b36645c-de5e-4263-ae06-2f6ff6a9add2',
            'lowercase_indication': 'plaque psoriasis',
            'indication': "plaque psoriasis",

        })

        self.indication2 = self.inst.create({
            'set_id': 'b5cdf4fd-9c95-441a-83eb-fbdb71ee42fa',
            'lowercase_indication': 'asthma',
            'indication': "asthma",
        })

        self.valid_data = {
            'set_id': '543dda2c-7fce-4cd3-9d0a-0a167c71f223',
            'lowercase_indication': "ulcerative colitis",
            'indication': "ulcerative colitis",
        }

    @parameterized.expand([
        ('set_id',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        expected_data = {
            **self.valid_data,
        }
        second_equals_first(expected_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

    @parameterized.expand([
        ('id', 'indication1', 'id', 1),
        ('indication', 'indication1', 'indication', 2),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('set_id',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.indication1[field_name],
        })
        second_equals_first(
            self.indication1,
            resp,
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'set_id': self.valid_data['set_id'],
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(2, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            **self.valid_data,
            **{'set_id': self.indication1['set_id'],
               'lowercase_indication': self.indication1[
                   'lowercase_indication'],
               'indication': self.indication1['indication']
               },
        }
        resp = self.inst.upsert(data)
        expected_data = {
            **data,
        }
        second_equals_first(
            expected_data,
            resp,
        )
        self.assertEqual(2, len(self.inst.read({})))



from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime
from marshmallow import ValidationError

from ...database import db_session
from ...resources.PatentIdentifierSynonym.models import (
    PatentIdentifierSynonymModel,
)
from .schemas import (
    PatentIdentifierSynonymResourceSchema,
    PatentIdentifierSynonymQueryParamsSchema,
)


schema_resource = PatentIdentifierSynonymResourceSchema()
schema_params = PatentIdentifierSynonymQueryParamsSchema()


class DBException(SQLAlchemyError):
    pass


class PatentIdentifierSynonymNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class PatentIdentifierSynonym:
    @staticmethod
    def create(params):
        """
        :param
            Dict: PatentIdentifierSynonymResourceSchema
        :return:
            PatentIdentifierSynonymResourceSchema
        :exception:
            ValidationError
            DBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            params: PatentIdentifierSynonymQuerySchema
        :return:
            List<PatentIdentifierSynonymResourceSchema>
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        patent_query = _build_query(params=data)
        response = schema_resource.dump(patent_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        :param
            params: PatentIdentifierSynonymQuerySchema
        :return:
            PatentIdentifierSynonymResourceSchema
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        patent_query = _build_query(params=data).one()
        response = schema_resource.dump(patent_query).data
        return response


def _helper_create(data):
    new_patent = PatentIdentifierSynonymModel(
        updated_at=datetime.utcnow(),
        **data,
    )
    try:
        db_session.add(new_patent)
        db_session.commit()
        patent_query = db_session.query(
            PatentIdentifierSynonymModel).get(new_patent.id)
        response = schema_resource.dump(patent_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _build_query(params):
    q = db_session.query(PatentIdentifierSynonymModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('patent_id'):
        q = q.filter_by(patent_id=params.get('patent_id'))
    if params.get('synonym'):
        q = q.filter_by(synonym=params.get('synonym'))
    return q



from marshmallow import (
    Schema,
    fields,
    validate,
)


class PharmacologicResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    pharma_set_id = fields.String(validate=not_blank, required=True)
    unii_code = fields.String(allow_none=True)
    unii_name = fields.String(allow_none=True)
    nui = fields.String(allow_none=True)
    code_system = fields.String(allow_none=True)
    class_name = fields.String(allow_none=True)
    class_type = fields.String(allow_none=True)
    updated_at = fields.DateTime()


class PharmacologicQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    pharma_set_id = fields.String(validate=not_blank)
    nui = fields.String(validate=not_blank)
    class_name = fields.String(validate=not_blank)


class PharmacologicPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    pharma_set_id = fields.String(validate=not_blank)
    unii_code = fields.String(allow_none=True)
    unii_name = fields.String(allow_none=True)
    nui = fields.String(allow_none=True)
    code_system = fields.String(allow_none=True)
    class_name = fields.String(allow_none=True)
    class_type = fields.String(allow_none=True)



from sqlalchemy.exc import SQLAlchemyError
from marshmallow import ValidationError

from cs_models.utils.utils import update_model
from ...database import db_session
from ...resources.Pharmacologic.models import PharmacologicModel
from .schemas import (
    PharmacologicPatchSchema,
    PharmacologicQueryParamsSchema,
    PharmacologicResourceSchema,
)


schema_resource = PharmacologicResourceSchema()
schema_params = PharmacologicQueryParamsSchema()
schema_patch = PharmacologicPatchSchema()


class Pharmacologic(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(PharmacologicResourceSchema)

        Returns: PharmacologicResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(PharmacologicQueryParamsSchema)

        Returns: List<PharmacologicResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        pharmacologic_query = _build_query(params=data)
        response = schema_resource.dump(pharmacologic_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(PharmacologicQueryParamsSchema)

        Returns: PharmacologicResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        pharmacologic_query = _build_query(params=data).one()
        response = schema_resource.dump(pharmacologic_query).data
        return response

    @staticmethod
    def update(id, params):
        """

        Args:
            id: int
            params: PharmacologicPatchSchema

        Returns:
            PharmacologicResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        pharmacologic_query = db_session.query(PharmacologicModel).filter_by(
            id=id).one()
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, pharmacologic_query)
        return response

    @staticmethod
    def delete(id):
        """
        Args:
            id: int

        Returns: string

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        ndc_query = db_session.query(PharmacologicModel).filter_by(id=id).one()
        try:
            db_session.delete(ndc_query)
            db_session.commit()
            db_session.close()
            return "Successfully deleted"
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise


def _helper_create(data):
    new_pharmacologic = PharmacologicModel(**data)
    try:
        db_session.add(new_pharmacologic)
        db_session.commit()
        pharmacologic_query = db_session.query(PharmacologicModel).get(
            new_pharmacologic.id)
        response = schema_resource.dump(pharmacologic_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, pharmacologic_query):
    data["id"] = pharmacologic_query.id
    try:
        update_model(data, pharmacologic_query)
        db_session.commit()
        response = schema_resource.dump(pharmacologic_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(PharmacologicModel)
    if params.get("id"):
        q = q.filter_by(id=params.get("id"))
    if params.get("pharma_set_id"):
        q = q.filter_by(pharma_set_id=params.get("pharma_set_id"))
    if params.get("nui"):
        q = q.filter_by(nui=params.get("nui"))
    if params.get("class_name"):
        q = q.filter_by(class_name=params.get("class_name"))
    return q



from sqlalchemy import (
    Column,
    String,
)

from ...database import Base


class NCTIdICDCrossModel(Base):
    __tablename__ = "nct_id_icd"

    nct_id = Column(String(128), nullable=False, primary_key=True)
    icd_code = Column(String(128), nullable=False, primary_key=True)



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
    ForeignKey,
    Boolean,
)

from ...database import Base


class SRCIdInterventionModel(Base):
    __tablename__ = "src_interventions"

    id = Column(Integer, primary_key=True)
    src_id = Column(String(128), nullable=False, index=True)
    src_type = Column(String(50), nullable=False, index=True)
    intervention_id = Column(
        Integer,
        ForeignKey('interventions.id'),
        nullable=False,
    )
    narrow = Column(Boolean, nullable=False)
    group = Column(Integer, nullable=False)
    is_deleted = Column(Boolean, nullable=True)
    score = Column(Integer, nullable=True)
    final_mapping = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    __table_args__ = (UniqueConstraint("src_id", "intervention_id"),)



from marshmallow import ValidationError
from parameterized import parameterized
from sqlalchemy.exc import IntegrityError

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.Patent import (
    Patent,
    PatentNotFoundException,
)


class PatentResourceTestCase(TestCase):
    def setUp(self):
        super(PatentResourceTestCase, self).setUp()
        self.inst = Patent()
        self.patent1 = self.inst.create({
            'patent_number': '1000',
            'jurisdiction': 'USPAT',
            'app_grp_art_number': '5000',
            'app_type': 'Utility',
            'grant_date': '2015/10/11',
            'app_status_date': '2015/10/13',
            'patent_pdf_url': 'https://pimg-fpiw.uspto.gov/fdd/36/39.pdf',
            'filing_date_us': '2015/10/05',
            'expiration_date': '2025/10/13',
            'espace_url': 'https://some-espace.com',
        })

    def test_create_validation_error_missing_fields(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {},
        )

    def test_create_violates_unique_constraint(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {
                'patent_number': self.patent1['patent_number'],
                'jurisdiction': self.patent1['jurisdiction'],
                'app_grp_art_number': '123',
            },
        )

    def test_create_base(self):
        data = {
            'patent_number': '2000',
            'jurisdiction': 'USPAT',
            'app_grp_art_number': '2222',
        }
        response = self.inst.create(data)
        second_equals_first(data, response)

    def test_create(self):
        data = {
            'patent_number': '2000',
            'jurisdiction': 'USPAT',
            'app_grp_art_number': '2222',
            'primary_identifier': 'US2000',
            'grant_date': '2017/03/01',
        }
        response = self.inst.create(data)
        self.assertEqual(data['patent_number'], response['patent_number'])
        self.assertEqual(
            data['app_grp_art_number'],
            response['app_grp_art_number'],
        )
        self.assertEqual(
            data['primary_identifier'],
            response['primary_identifier'],
        )
        self.assertEqual(
            '2017-03-01T00:00:00+00:00',
            response['grant_date'],
        )

    def test_read_validation_error_blank_app_grp_art_number(self):
        self.assertRaises(
            ValidationError,
            self.inst.read,
            {'app_grp_art_number': ''},
        )

    def test_read_all(self):
        response = self.inst.read({})
        self.assertEqual(len(response), 1)

    def test_read_w_params(self):
        # setup
        self.inst.create({
            'patent_number': '2000',
            'jurisdiction': 'USPAT',
            'app_grp_art_number': '2222',
            'primary_identifier': 'US2000',
        })

        response = self.inst.read({})
        self.assertEqual(len(response), 2)

        response = self.inst.read({'patent_number': '1000'})
        self.assertEqual(len(response), 1)

        response = self.inst.read({'app_grp_art_number': '2222'})
        self.assertEqual(len(response), 1)

        response = self.inst.read({'primary_identifier': 'US2000'})
        self.assertEqual(len(response), 1)

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            PatentNotFoundException,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        data = self.inst.read({'id': self.patent1['id']})
        self.assertEqual(len(data), 1)
        response = self.inst.delete(id=self.patent1['id'])
        self.assertEqual(response, 'Successfully deleted')
        data = self.inst.read({'id': self.patent1['id']})
        self.assertEqual(len(data), 0)

    @parameterized.expand([
        ('patent_number',),
        ('jurisdiction',),
        ('app_grp_art_number',),
        ('app_type',),
        ('country_code',),
        ('document_number',),
        ('kind_code',),
        ('primary_identifier',),
        ('abstract_text',),
        ('applicant',),
        ('inventors',),
        ('title',),
        ('url',),
        ('pto_adjustments',),
        ('appl_delay',),
        ('total_pto_days',),
        ('assignee',),
        ('patent_pdf_url',),
        ('espace_url',),
    ])
    def test_update(self, field):
        new_data = {
            field: 'something new',
        }
        resp = self.inst.update(
            id=self.patent1['id'],
            params=new_data,
        )
        expected_result = {
            **self.patent1,
            **new_data,
            # the following fields should not be updated
            **{
                'patent_number': self.patent1['patent_number'],
                'jurisdiction': self.patent1['jurisdiction'],
            },
        }

        second_equals_first(
            expected_result,
            resp,
            fields_to_ignore=['updated_at'],
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {'app_grp_art_number': ''},
        )

    def test_upsert_creates_new_entry(self):
        data = {
            'patent_number': '21321',
            'jurisdiction': 'USPAT',
            'app_grp_art_number': '123123',
        }
        self.assertEqual(1, len(self.inst.read({})))
        resp = self.inst.upsert(data)
        self.assertEqual(2, len(self.inst.read({})))
        second_equals_first(
            data,
            resp,
        )

    def test_upsert_updates_existing_row(self):
        data = {
            'patent_number': '21321',
            'jurisdiction': 'USPAT',
            'app_grp_art_number': '123123',
            'kind_code': 'something',
        }
        self.assertEqual(1, len(self.inst.read({})))
        resp = self.inst.upsert(data)
        self.assertEqual(2, len(self.inst.read({})))
        second_equals_first(
            data,
            resp,
        )

        new_data = {
            'patent_number': resp['patent_number'],
            'jurisdiction': resp['jurisdiction'],
            'app_grp_art_number': '123123',
            'abstract_text': 'Some abstract text',
        }
        response = self.inst.upsert(params=new_data)
        self.assertEqual(2, len(self.inst.read({})))
        expected_data = {
            **data,
            **new_data,
        }
        second_equals_first(
            expected_data,
            response,
        )

    @parameterized.expand([
        ('-',),
        (None,),
    ])
    def test_upsert_empty_date_field(self, empty_value):
        data = {
            'patent_number': '21321',
            'jurisdiction': 'USPAT',
            'app_grp_art_number': '123123',
            'grant_date': empty_value,
        }
        self.assertEqual(1, len(self.inst.read({})))
        resp = self.inst.upsert(data)
        self.assertEqual(2, len(self.inst.read({})))
        second_equals_first(
            data,
            resp,
        )

    @parameterized.expand([
        ('1599', 0),
        ('1600', 1),
        ('1601', 1),
        ('16100', 0),
        ('1700', 0),
        ('1701', 0),
        ('12316', 0),
    ])
    def test_pharma_patents(self, app_grp_art_number, expected_length):
        self.inst.create({
            'patent_number': '2000',
            'jurisdiction': 'USPAT',
            'app_grp_art_number': app_grp_art_number,
        })
        pharma_patents = self.inst.pharma_patents()
        self.assertEqual(expected_length, len(pharma_patents))



from marshmallow import (
    Schema,
    fields,
)


class ClaimDependencyResourceSchema(Schema):
    id = fields.Integer(dump_only=True)
    claim_id = fields.Integer(required=True)
    parent_claim_id = fields.Integer(required=True)
    updated_at = fields.DateTime()


class ClaimDependencyQueryParamsSchema(Schema):
    id = fields.Integer()
    claim_id = fields.Integer()
    parent_claim_id = fields.Integer()



from marshmallow import (
    Schema,
    fields,
    validate,
)


class CompanyConditionStageResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    company_condition_stage = fields.String(allow_none=False, validate=not_blank)
    stage = fields.String(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)



import copy
from marshmallow import ValidationError
from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.Pharmacologic import Pharmacologic


class PharmacologicResourceTestCase(TestCase):
    def setUp(self):
        super(PharmacologicResourceTestCase, self).setUp()

        self.inst = Pharmacologic()

        self.pharmacologic1 = self.inst.create({
            "pharma_set_id": "ab7b3796-63bc-4891-91bd-232fbeefa4b1",
            'unii_code': "2A41N28TJQ",
            'unii_name': 'BROMUS CARINATUS POLLEN',
            "nui": "N0000185367",
            "code_system": "2.16.840.1.113883.6.345",
            "class_name": "Non-Standardized Pollen Allergenic Extract",
            "class_type": "EPC",
        })

        self.pharmacologic2 = self.inst.create({
            "pharma_set_id": "901fd38a-6b75-42b6-97e4-0b7d9cab8a27",
            'unii_code': "OG645J8RVW",
            'unii_name': 'PIRBUTEROL',
            "nui": "N0000009922",
            "code_system": "2.16.840.1.113883.3.26.1.5",
            "class_name": "Adrenergic beta2-Agonists",
            "class_type": "MoA",
        })

        self.valid_data = {
            "pharma_set_id": "7df933d0-1a3d-4280-82de-c10e9305fadd",
            'unii_code': "2GQ1IRY63P",
            'unii_name': 'ATRACURIUM',
            "nui": "N0000175732",
            "code_system": "2.16.840.1.113883.3.26.1.5",
            "class_name": "Neuromuscular Nondepolarizing Blockade",
            "class_type": "PE",
        }

    @parameterized.expand([
        ('pharma_set_id',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        expected_data = {
            **self.valid_data,
        }
        second_equals_first(expected_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

    @parameterized.expand([
        ('id', 'pharmacologic1', 'id', 1),
        ('pharma_set_id', 'pharmacologic1', 'pharma_set_id', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('pharma_set_id',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.pharmacologic1[field_name],
        })
        second_equals_first(
            self.pharmacologic1,
            resp,
        )



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    ForeignKey,
    DateTime,
)

from ...database import Base


class StreamModel(Base):
    __tablename__ = "stream"

    id = Column(Integer, primary_key=True)
    date = Column(DateTime, nullable=False)
    resource_type = Column(String(128), nullable=False)
    norm_cui = Column(String(50), nullable=True, index=True)
    condition_id = Column(
        Integer,
        ForeignKey('conditions.id'),
        nullable=True,
    )
    company_sec_id = Column(
        Integer,
        ForeignKey('companies_sec.id'),
        nullable=True,
    )
    company_ous_id = Column(
        Integer,
        ForeignKey('companies_ous.id'),
        nullable=True,
    )
    ptab2_proceeding_id = Column(
        Integer,
        ForeignKey('ptab2_proceedings.id'),
        nullable=True,
    )
    source_table = Column(String(50), nullable=False)
    source_table_id = Column(Integer, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class DrugClassResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    disease_id = fields.String(validate=not_blank, required=True)
    disease_name = fields.String(validate=not_blank, required=True)
    moa = fields.String(allow_none=True)
    moa_id = fields.String(validate=not_blank, required=True)
    epc = fields.String(allow_none=True)
    epc_id = fields.String(validate=not_blank, required=True)
    struct = fields.String(allow_none=True)
    struct_id = fields.String(allow_none=True)
    pe = fields.String(allow_none=True)
    pe_id = fields.String(allow_none=True)
    updated_at = fields.DateTime()



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
    UniqueConstraint,
)

from ...database import Base


class PharmActionModel(Base):
    __tablename__ = "pharm_actions"

    id = Column(Integer, primary_key=True)
    pharm_action_id = Column(String(128), nullable=False)
    pharm_action = Column(Text, nullable=False)
    pharm_action_type = Column(String(50), nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    __table_args__ = (UniqueConstraint("pharm_action_id"),)



from sqlalchemy import (
    Column,
    Integer,
    Text,
    DateTime,
    ForeignKey,
    UniqueConstraint,
)
from datetime import datetime

from ...database import Base


class ClaimModel(Base):
    __tablename__ = 'claims'

    id = Column(Integer, primary_key=True)
    patent_id = Column(
        Integer,
        ForeignKey('patents.id'),
    )
    patent_application_id = Column(
        Integer,
        ForeignKey('patent_applications.id'),
    )
    claim_number = Column(Integer, nullable=False)
    claim_text = Column(Text, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (
        UniqueConstraint('patent_id', 'claim_number'),
        UniqueConstraint('patent_application_id', 'claim_number'),
    )



from marshmallow import (
    Schema,
    fields,
)


class DocumentTreeResourceSchema(Schema):
    file_id = fields.Integer(required=True)
    value = fields.Dict(required=True)
    updated_at = fields.DateTime()


class DocumentTreeQueryParamsSchema(Schema):
    file_id = fields.Integer()


class DocumentTreePatchSchema(Schema):
    file_id = fields.Integer(required=True)
    value = fields.Dict(required=True)



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
)
from datetime import datetime

from ...database import Base


class ConfigModel(Base):
    __tablename__ = 'config'

    id = Column(Integer, primary_key=True)
    table_name = Column(String(128), nullable=False)
    field_name = Column(String(128), nullable=False)
    latest_date = Column(DateTime, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint('table_name', 'field_name'),)



from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
    MultipleResultsFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.PacerCase import PacerCase


class PacerCaseResourceTestCase(TestCase):
    def setUp(self):
        super(PacerCaseResourceTestCase, self).setUp()
        self.inst = PacerCase()

        self.pacer_case1 = self.inst.create({
            'case_no': '2:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '1000',
            'cause': '15:1126 Patent Infringement',
            'county': 'Union',
            'defendant': "DR. REDDY'S LABORATORIES, LTD.",
            'disposition': "Something",
            'filed_date': '07/20/2017',
            'flags': 'ANDA,SCHEDO',
            'jurisdiction': 'Federal Question',
            'lead_case': 'None',
            'nature_of_suit': '835',
            'office': 'Newark',
            'plaintiff': 'CELGENE CORPORATION',
            'related_case': '2:16-cv-07704-SDW-LDW',
            'terminated_date': '09/20/2017',
        })

        self.pacer_case2 = self.inst.create({
            'case_no': '3:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '2000',
            'cause': '15:1126 Patent Infringement',
        })

    @parameterized.expand([
        ('case_no',),
        ('court_id',),
        ('pacer_case_external_id',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = {
            'case_no': '3:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '351755',
            'cause': '15:1126 Patent Infringement',
        }
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create_violates_unique_constraint(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {
                'case_no': '4:17-cv-05314-SDW-LDW',
                'court_id': self.pacer_case1['court_id'],
                'pacer_case_external_id': (
                    self.pacer_case1['pacer_case_external_id']),
                'cause': '15:1126 Patent Infringement',
            },
        )

    def test_create(self):
        data = {
            'case_no': '100:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '10000',
            'cause': '15:1126 Patent Infringement',
        }
        resp = self.inst.create(data)
        second_equals_first(data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

    @parameterized.expand([
        ('id', 'pacer_case1', 'id', 1),
        ('case_no', 'pacer_case1', 'case_no', 1),
        ('court_id', 'pacer_case1', 'court_id', 2),
        ('pacer_case_external_id', 'pacer_case1', 'pacer_case_external_id', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    def test_read_w_params_ids(self):
        resp = self.inst.read({'ids': []})
        self.assertEqual(0, len(resp))

        resp = self.inst.read({'ids': [self.pacer_case1['id']]})
        self.assertEqual(1, len(resp))
        second_equals_first(
            self.pacer_case1,
            resp[0]
        )

    @parameterized.expand([
        ('id', 999, NoResultFound),
        ('court_id', 'njd', MultipleResultsFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('case_no',),
        ('pacer_case_external_id',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.pacer_case1[field_name],
        })
        second_equals_first(
            self.pacer_case1,
            resp,
        )

    def test_update(self):
        new_data = {
            'case_no': 'something new',
            'court_id': 'ny',
            'pacer_case_external_id': '9999',
            'cause': 'something new',
        }
        resp = self.inst.update(
            id=self.pacer_case1['id'],
            params=new_data,
        )
        expected_result = {
            **self.pacer_case1,
            **new_data,
            # the following fields cannot be updated
            **{
                'court_id': self.pacer_case1['court_id'],
                'pacer_case_external_id': (
                    self.pacer_case1['pacer_case_external_id']),
            },
        }

        expected_result.pop('updated_at')
        resp.pop('updated_at')
        second_equals_first(
            expected_result,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({'id': self.pacer_case1['id']})
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.pacer_case1['id']},
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'case_no': '3:17-cv-05314-SDW-LDW',
                'pacer_case_external_id': '351755',
                'cause': '15:1126 Patent Infringement',
            }
        )

    def test_upsert_creates_new_entry(self):
        data = {
            'case_no': '3:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '351755',
            'cause': '',
            'county': 'Union',
            'defendant': "DR. REDDY'S LABORATORIES, LTD.",
            'disposition': "Something",
            'filed_date': None,
            'flags': 'ANDA,SCHEDO',
            'jurisdiction': 'Federal Question',
            'lead_case': 'None',
            'nature_of_suit': '835',
            'office': 'Newark',
            'plaintiff': 'CELGENE CORPORATION',
            'related_case': '2:16-cv-07704-SDW-LDW',
            'terminated_date': None,
        }
        self.assertEqual(2, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            'case_no': '3:17-cv-05314-SDW-LDW',
            'court_id': self.pacer_case1['court_id'],
            'pacer_case_external_id': (
                self.pacer_case1['pacer_case_external_id']),
            'cause': '15:1126 Patent Infringement',
        }
        resp = self.inst.upsert(data)
        second_equals_first(
            data,
            resp,
        )
